# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: ean2.py - Last Update: 11/15/2024 Ver. 2.12.0 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals, generators, with_statement, nested_scopes
from upcean.predraw import *
import re
import sys
import upcean.support
try:
    file
except NameError:
    from io import IOBase as file
try:
    from io import StringIO, BytesIO
except ImportError:
    try:
        from cStringIO import StringIO
        from cStringIO import StringIO as BytesIO
    except ImportError:
        from StringIO import StringIO
        from StringIO import StringIO as BytesIO
pilsupport = upcean.support.check_for_pil()
pillowsupport = upcean.support.check_for_pillow()
cairosupport = upcean.support.check_for_cairo()
qahirahsupport = upcean.support.check_for_qahirah()
cairosvgsupport = upcean.support.check_for_cairosvg()
svgwritesupport = upcean.support.check_for_svgwrite()
wandsupport = upcean.support.check_for_wand()
magicksupport = upcean.support.check_for_magick()
defaultdraw = upcean.support.defaultdraw
if(pilsupport or pillowsupport):
    import upcean.predraw.prepil
if(cairosupport):
    import upcean.predraw.precairo
if(qahirahsupport):
    import upcean.predraw.preqahirah
if(svgwritesupport):
    import upcean.predraw.presvgwrite
if(wandsupport):
    import upcean.predraw.prewand
if(magicksupport):
    import upcean.predraw.premagick

def get_ean2_barcode_size(upc, resize=1, shiftxy=(0, 0), barheight=(48, 54), barwidth=(1, 1)):
    barheightadd = barheight[1]
    if(barheight[0] >= barheight[1]):
        barheightadd = barheight[0] + 6
    else:
        barheightadd = barheight[1]
    if(len(upc) > 2 or len(upc) < 2):
        return False
    upc_matches = re.findall("(\\d{2})", upc)
    if(len(upc_matches) <= 0):
        return False
    if(not re.findall("^([0-9]*[\\.]?[0-9])", str(resize)) or int(resize) < 1):
        resize = 1
    upc_size_add = 0
    reswoshift = (((29 * barwidth[0]) + upc_size_add) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize))
    reswshift = ((((29 + shiftxy[0]) * barwidth[0]) + upc_size_add) * int(resize), ((barheightadd + shiftxy[1]) + (9 * barwidth[1])) * int(resize))
    return {'without_shift': reswoshift, 'with_shift': reswshift}


def encode_ean2_barcode(inimage, upc, resize=1, shiftxy=(0, 0), barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False)):
    upc = str(upc)
    hidesn = hideinfo[0]
    hidecd = hideinfo[1]
    hidetext = hideinfo[2]
    barheightadd = barheight[1]
    if(barheight[0] >= barheight[1]):
        barheightadd = barheight[0] + 6
    else:
        barheightadd = barheight[1]
    if(inimage is None):
        upc_img = None
        upc_preimg = None
    else:
        upc_img = inimage[0]
        upc_preimg = inimage[1]
    imageoutlib = None
    if pilsupport and isinstance(upc_img, ImageDraw.ImageDraw) and isinstance(upc_preimg, Image.Image):
        imageoutlib = "pillow"
    elif cairosupport and isinstance(upc_img, cairo.Context) and isinstance(upc_preimg, cairo.Surface):
        imageoutlib = "cairo"
    elif qahirahsupport and isinstance(upc_img, qah.Context) and isinstance(upc_preimg, qah.Surface):
        imageoutlib = "qahirah"
    elif svgwritesupport and isinstance(upc_img, svgwrite.Drawing):
        imageoutlib = "svgwrite"
    elif wandsupport and isinstance(upc_img, wImage):
        imageoutlib = "wand"
    elif magicksupport and isinstance(upc_img, PythonMagick.Image):
        imageoutlib = "magick"
    elif(imageoutlib != "pillow" and imageoutlib != "cairo" and imageoutlib != "qahirah" and imageoutlib != "cairosvg" and imageoutlib != "svgwrite" and imageoutlib != "wand" and imageoutlib != "magick" and inimage != "none" and inimage is not None):
        imageoutlib = None
    elif(inimage == "none" or inimage is None):
        imageoutlib = None
    elif(not pilsupport and not cairosupport and not svgwritesupport):
        return False
    else:
        return False
    if(len(upc) > 2 or len(upc) < 2):
        return False
    upc_matches = re.findall("(\\d{2})", upc)
    if(len(upc_matches) <= 0):
        return False
    if(not re.findall("^([0-9]*[\\.]?[0-9])", str(resize)) or int(resize) < 1):
        resize = 1
    if(pilsupport and imageoutlib == "pillow"):
        pil_addon_fix = 0
        cairo_addon_fix = 0
    elif((cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")) or (svgwritesupport and cairosvgsupport and imageoutlib == "svgwrite") or (qahirahsupport and imageoutlib == "qahirah")):
        pil_addon_fix = 0
        cairo_addon_fix = (9 * (int(resize) * barwidth[1]))
    elif((wandsupport and imageoutlib == "wand") or (magicksupport and imageoutlib == "magick")):
        pil_addon_fix = 0
        cairo_addon_fix = (10 * (int(resize) * barwidth[1]))
    elif(svgwritesupport and imageoutlib == "svgwrite"):
        pil_addon_fix = 0
        cairo_addon_fix = (8 * (int(resize) * barwidth[1]))
    else:
        pil_addon_fix = 0
        cairo_addon_fix = 0
    cairo_addon_fix += (shiftxy[1] * (int(resize) * barwidth[1]))
    CheckSum = int(upc_matches[0]) % 4
    LeftDigit = list(upc_matches[0])
    upc_size_add = 0
    if(inimage is not None):
        drawColorRectangle(upc_img, 0 + (shiftxy[0] * barwidth[0]) * int(resize), 0 + (shiftxy[1] * barwidth[1]) * int(resize), (((29 + shiftxy[0]) * barwidth[0]) + upc_size_add) * int(resize), ((barheightadd + shiftxy[1]) + (9 * barwidth[1])) * int(resize), barcolor[2], imageoutlib)
    upc_array = {'upc': upc, 'heightadd': 9, 'type': "ean2", 'barsize': [], 'code': [], 'text': {'location': [], 'text': [], 'type': []}}
    LineSize = (barheight[0] + shiftxy[1]) * int(resize)
    if(hidetext):
        LineSize = (barheight[1] + shiftxy[1]) * int(resize)
    upc_array['code'].append([0, 1, 0, 1, 1])
    start_barcode = [0, 1, 0, 1, 1]
    LineStart = (shiftxy[0] * barwidth[0]) * int(resize)
    BarNum = 0
    start_bc_num_end = len(start_barcode)
    barsizeloop = []
    LineSizeType = 0
    while(BarNum < start_bc_num_end):
        if(start_barcode[BarNum] == 1):
            drawColorLine(upc_img, LineStart, (10 + shiftxy[1]) * int(resize), LineStart,
                          LineSize, barwidth[0] * int(resize), barcolor[0], imageoutlib)
        if(start_barcode[BarNum] == 0):
            drawColorLine(upc_img, LineStart, (10 + shiftxy[1]) * int(resize), LineStart,
                          LineSize, barwidth[0] * int(resize), barcolor[2], imageoutlib)
        barsizeloop.append(LineSizeType)
        LineStart += barwidth[0] * int(resize)
        BarNum += 1
    upc_array['barsize'].append(barsizeloop)
    NumZero = 0
    LineSizeType = 0
    while (NumZero < len(LeftDigit)):
        left_barcolor_l = [0, 0, 0, 0, 0, 0, 0]
        left_barcolor_g = [1, 1, 1, 1, 1, 1, 1]
        if(int(LeftDigit[NumZero]) == 0):
            left_barcolor_l = [0, 0, 0, 1, 1, 0, 1]
            left_barcolor_g = [0, 1, 0, 0, 1, 1, 1]
        if(int(LeftDigit[NumZero]) == 1):
            left_barcolor_l = [0, 0, 1, 1, 0, 0, 1]
            left_barcolor_g = [0, 1, 1, 0, 0, 1, 1]
        if(int(LeftDigit[NumZero]) == 2):
            left_barcolor_l = [0, 0, 1, 0, 0, 1, 1]
            left_barcolor_g = [0, 0, 1, 1, 0, 1, 1]
        if(int(LeftDigit[NumZero]) == 3):
            left_barcolor_l = [0, 1, 1, 1, 1, 0, 1]
            left_barcolor_g = [0, 1, 0, 0, 0, 0, 1]
        if(int(LeftDigit[NumZero]) == 4):
            left_barcolor_l = [0, 1, 0, 0, 0, 1, 1]
            left_barcolor_g = [0, 0, 1, 1, 1, 0, 1]
        if(int(LeftDigit[NumZero]) == 5):
            left_barcolor_l = [0, 1, 1, 0, 0, 0, 1]
            left_barcolor_g = [0, 1, 1, 1, 0, 0, 1]
        if(int(LeftDigit[NumZero]) == 6):
            left_barcolor_l = [0, 1, 0, 1, 1, 1, 1]
            left_barcolor_g = [0, 0, 0, 0, 1, 0, 1]
        if(int(LeftDigit[NumZero]) == 7):
            left_barcolor_l = [0, 1, 1, 1, 0, 1, 1]
            left_barcolor_g = [0, 0, 1, 0, 0, 0, 1]
        if(int(LeftDigit[NumZero]) == 8):
            left_barcolor_l = [0, 1, 1, 0, 1, 1, 1]
            left_barcolor_g = [0, 0, 0, 1, 0, 0, 1]
        if(int(LeftDigit[NumZero]) == 9):
            left_barcolor_l = [0, 0, 0, 1, 0, 1, 1]
            left_barcolor_g = [0, 0, 1, 0, 1, 1, 1]
        left_barcolor = left_barcolor_l
        if(CheckSum == 0 and NumZero == 0):
            left_barcolor = left_barcolor_l
        if(CheckSum == 0 and NumZero == 1):
            left_barcolor = left_barcolor_l
        if(CheckSum == 1 and NumZero == 0):
            left_barcolor = left_barcolor_l
        if(CheckSum == 1 and NumZero == 1):
            left_barcolor = left_barcolor_g
        if(CheckSum == 2 and NumZero == 0):
            left_barcolor = left_barcolor_g
        if(CheckSum == 2 and NumZero == 1):
            left_barcolor = left_barcolor_l
        if(CheckSum == 3 and NumZero == 0):
            left_barcolor = left_barcolor_g
        if(CheckSum == 3 and NumZero == 1):
            left_barcolor = left_barcolor_g
        upc_array['code'].append(left_barcolor)
        InnerUPCNum = 0
        barsizeloop = []
        while (InnerUPCNum < len(left_barcolor)):
            if(left_barcolor[InnerUPCNum] == 1):
                drawColorLine(upc_img, LineStart, (10 + shiftxy[1]) * int(resize), LineStart,
                              LineSize, barwidth[0] * int(resize), barcolor[0], imageoutlib)
            if(left_barcolor[InnerUPCNum] == 0):
                drawColorLine(upc_img, LineStart, (10 + shiftxy[1]) * int(resize), LineStart,
                              LineSize, barwidth[0] * int(resize), barcolor[2], imageoutlib)
            LineStart += barwidth[0] * int(resize)
            barsizeloop.append(LineSizeType)
            BarNum += 1
            InnerUPCNum += 1
        upc_array['barsize'].append(barsizeloop)
        if(NumZero == 0):
            drawColorLine(upc_img, LineStart, (10 + shiftxy[1]) * int(resize), LineStart,
                          LineSize, barwidth[0] * int(resize), barcolor[2], imageoutlib)
            LineStart += barwidth[0] * int(resize)
            upc_array['barsize'].append([LineSizeType])
            upc_array['code'].append([0])
            BarNum += 1
            drawColorLine(upc_img, LineStart, (10 + shiftxy[1]) * int(resize), LineStart,
                          LineSize, barwidth[0] * int(resize), barcolor[0], imageoutlib)
            LineStart += barwidth[0] * int(resize)
            upc_array['barsize'].append([LineSizeType])
            upc_array['code'].append([1])
            BarNum += 1
        NumZero += 1
    upc_array['code'].append([0, 0, 0, 0, 0, 0, 0, 0, 0])
    end_barcode = [0, 0, 0, 0, 0, 0, 0, 0, 0]
    end_bc_num = 0
    end_bc_num_end = len(end_barcode)
    LineSize = (barheight[1] + shiftxy[1]) * int(resize)
    barsizeloop = []
    LineSizeType = 0
    while(end_bc_num < end_bc_num_end):
        if(end_barcode[end_bc_num] == 1):
            drawColorLine(upc_img, LineStart, (10 + shiftxy[1]) * int(resize), LineStart,
                          LineSize, barwidth[0] * int(resize), barcolor[0], imageoutlib)
        if(end_barcode[end_bc_num] == 0):
            drawColorLine(upc_img, LineStart, (10 + shiftxy[1]) * int(resize), LineStart,
                          LineSize, barwidth[0] * int(resize), barcolor[2], imageoutlib)
        barsizeloop.append(LineSizeType)
        end_bc_num += 1
        LineStart += barwidth[0] * int(resize)
        BarNum += 1
    upc_array['barsize'].append(barsizeloop)
    if(not hidetext):
        if(svgwritesupport and imageoutlib == "svgwrite"):
            try:
                upcean.predraw.presvgwrite.embed_font(upc_img, fontpathocrb, "OCRB")
            except OSError:
                upcean.predraw.presvgwrite.embed_font(upc_img, fontpathocrbalt, "OCRB")
    NumTxtZero = 0
    LineTxtStart = ((shiftxy[0] + 6) * int(resize))
    LineTxtStartNorm = 6
    upc_print = LeftDigit
    while (NumTxtZero < len(upc_print)):
        texthidden = False
        if hidetext:
            texthidden = True
        if(not texthidden):
            drawColorText(upc_img, 10 * int(resize * barwidth[1]), LineTxtStart * barwidth[0], cairo_addon_fix + (
            (barheight[0]) * int(resize)) + pil_addon_fix, upc_print[NumTxtZero], barcolor[1], "ocrb", imageoutlib)
        upc_array['text']['location'].append(LineTxtStartNorm)
        upc_array['text']['text'].append(upc_print[NumTxtZero])
        upc_array['text']['type'].append("txt")
        LineTxtStart += 7 * int(resize)
        LineTxtStartNorm += 7
        NumTxtZero += 1
    if((cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg"))):
        upc_preimg.flush()
    if(imageoutlib is None):
        return upc_array
    else:
        return [upc_img, upc_preimg, imageoutlib]

def draw_ean2sup_barcode(upc, resize=1, barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False), imageoutlib=defaultdraw):
    barheightadd = barheight[1]
    if(barheight[0] >= barheight[1]):
        barheightadd = barheight[0] + 6
    else:
        barheightadd = barheight[1]
    if(not pilsupport and imageoutlib == "pillow"):
        imageoutlib = "cairo"
    if(not cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
        imageoutlib = "pillow"
    if(not cairosupport and imageoutlib == "cairosvg"):
        imageoutlib = "pillow"
    if(not svgwritesupport and imageoutlib == "svgwrite"):
        imageoutlib = "pillow"
    if(imageoutlib != "pillow" and imageoutlib != "cairo" and imageoutlib != "qahirah" and imageoutlib != "cairosvg" and imageoutlib != "svgwrite"):
        imageoutlib = "pillow"
    upc_size_add = 0
    upc_img, upc_preimg = upcean.predraw.new_image_surface(((29 * barwidth[0]) + upc_size_add) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize), barcolor[2], imageoutlib)
    imgout = encode_ean2_barcode([upc_img, upc_preimg], upc, resize, (0, 0), barheight, barwidth, barcolor, hideinfo)
    return [upc_img, upc_preimg, imageoutlib]

def create_ean2sup_barcode(upc, outfile="./ean2.png", resize=1, barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False), imageoutlib=defaultdraw):
    if(not pilsupport and imageoutlib == "pillow"):
        imageoutlib = "svgwrite"
    if(not cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
        imageoutlib = "svgwrite"
    if(not qahirahsupport and imageoutlib == "qahirah"):
        imageoutlib = "svgwrite"
    if(not cairosupport and imageoutlib == "cairosvg"):
        imageoutlib = "svgwrite"
    if(not svgwritesupport and imageoutlib == "svgwrite"):
        imageoutlib = "svgwrite"
    if(not wandsupport and imageoutlib == "wand"):
        imageoutlib = "svgwrite"
    if(not magicksupport and imageoutlib == "magick"):
        imageoutlib = "svgwrite"
    if(imageoutlib != "pillow" and imageoutlib != "cairo" and imageoutlib != "qahirah" and imageoutlib != "cairosvg" and imageoutlib != "wand" and imageoutlib != "magick" and imageoutlib != "svgwrite"):
        imageoutlib = "svgwrite"
    if(not pilsupport and not cairosupport):
        imageoutlib = "svgwrite"
    if(outfile is None):
        if(imageoutlib == "cairosvg"):
            oldoutfile = None
            outfile = None
            outfileext = "SVG"
        else:
            oldoutfile = None
            outfile = None
            outfileext = None
    else:
        oldoutfile = upcean.predraw.get_save_filename(
            outfile, imageoutlib)
        if(isinstance(oldoutfile, tuple) or isinstance(oldoutfile, list)):
            del(outfile)
            outfile = oldoutfile[0]
            outfileext = oldoutfile[1]
            if(cairosupport and imageoutlib == "cairo" and outfileext == "SVG"):
                imageoutlib = "cairosvg"
            if(cairosupport and imageoutlib == "cairosvg" and outfileext != "SVG"):
                imageoutlib = "cairo"
    imgout = draw_ean2sup_barcode(upc, resize, barheight, barwidth, barcolor, hideinfo, imageoutlib)
    upc_img = imgout[0]
    upc_preimg = imgout[1]
    if(oldoutfile is None or isinstance(oldoutfile, bool)):
        return [upc_img, upc_preimg, imageoutlib]
    else:
        return upcean.predraw.save_to_file([upc_img, upc_preimg], outfile, outfileext, "ean2; "+upc, imageoutlib)
    return True

def draw_upc2sup_barcode(upc, resize=1, barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False), imageoutlib=defaultdraw):
    return draw_ean2sup_barcode(upc, resize, barheight, barwidth, barcolor, hideinfo, imageoutlib)

def create_upc2sup_barcode(upc, outfile="./upc2.png", resize=1, barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False), imageoutlib=defaultdraw):
    return create_ean2sup_barcode(upc, outfile, resize, barheight, barwidth, barcolor, hideinfo, imageoutlib)

def draw_ean2_barcode(upc, resize=1, barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False), imageoutlib=defaultdraw):
    barheightadd = barheight[1]
    if(barheight[0] >= barheight[1]):
        barheightadd = barheight[0] + 6
    else:
        barheightadd = barheight[1]
    if(not pilsupport and imageoutlib == "pillow"):
        imageoutlib = "cairo"
    if(not cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
        imageoutlib = "pillow"
    if(not cairosupport and imageoutlib == "cairosvg"):
        imageoutlib = "pillow"
    if(not svgwritesupport and imageoutlib == "svgwrite"):
        imageoutlib = "pillow"
    if(imageoutlib != "pillow" and imageoutlib != "cairo" and imageoutlib != "qahirah" and imageoutlib != "cairosvg" and imageoutlib != "svgwrite"):
        imageoutlib = "pillow"
    upc_size_add = 0
    upc_img, upc_preimg = upcean.predraw.new_image_surface(((29 * barwidth[0]) + upc_size_add) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize), barcolor[2], imageoutlib)
    drawColorRectangle(upc_img, 0, 0, (((29 + 8) * barwidth[0]) + upc_size_add) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize), barcolor[2], imageoutlib)
    imgout = encode_ean2_barcode([upc_img, upc_preimg], upc, resize, (8 * int(resize), 0), barheight, barwidth, barcolor, hideinfo)
    return [upc_img, upc_preimg, imageoutlib]

def create_ean2_barcode(upc, outfile="./ean2.png", resize=1, barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False), imageoutlib=defaultdraw):
    if(not pilsupport and imageoutlib == "pillow"):
        imageoutlib = "svgwrite"
    if(not cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
        imageoutlib = "svgwrite"
    if(not qahirahsupport and imageoutlib == "qahirah"):
        imageoutlib = "svgwrite"
    if(not cairosupport and imageoutlib == "cairosvg"):
        imageoutlib = "svgwrite"
    if(not svgwritesupport and imageoutlib == "svgwrite"):
        imageoutlib = "svgwrite"
    if(not wandsupport and imageoutlib == "wand"):
        imageoutlib = "svgwrite"
    if(not magicksupport and imageoutlib == "magick"):
        imageoutlib = "svgwrite"
    if(imageoutlib != "pillow" and imageoutlib != "cairo" and imageoutlib != "qahirah" and imageoutlib != "cairosvg" and imageoutlib != "wand" and imageoutlib != "magick" and imageoutlib != "svgwrite"):
        imageoutlib = "svgwrite"
    if(not pilsupport and not cairosupport):
        imageoutlib = "svgwrite"
    if(outfile is None):
        if(imageoutlib == "cairosvg"):
            oldoutfile = None
            outfile = None
            outfileext = "SVG"
        else:
            oldoutfile = None
            outfile = None
            outfileext = None
    else:
        oldoutfile = upcean.predraw.get_save_filename(
            outfile, imageoutlib)
        if(isinstance(oldoutfile, tuple) or isinstance(oldoutfile, list)):
            del(outfile)
            outfile = oldoutfile[0]
            outfileext = oldoutfile[1]
            if(cairosupport and imageoutlib == "cairo" and outfileext == "SVG"):
                imageoutlib = "cairosvg"
            if(cairosupport and imageoutlib == "cairosvg" and outfileext != "SVG"):
                imageoutlib = "cairo"
    imgout = draw_ean2_barcode(upc, resize, barheight, barwidth, barcolor, hideinfo, imageoutlib)
    upc_img = imgout[0]
    upc_preimg = imgout[1]
    if(oldoutfile is None or isinstance(oldoutfile, bool)):
        return [upc_img, upc_preimg, imageoutlib]
    else:
        return upcean.predraw.save_to_file([upc_img, upc_preimg], outfile, outfileext, "ean2; "+upc, imageoutlib)
    return True

def encode_upc2_barcode(inimage, upc, resize=1, shiftxy=(0, 0), barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False)):
    return encode_ean2_barcode(inimage, upc, resize, shiftxy, barheight, barwidth, barcolor, hideinfo)

def draw_upc2_barcode(upc, resize=1, barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False), imageoutlib=defaultdraw):
    return draw_ean2_barcode(upc, resize, barheight, barwidth, barcolor, hideinfo, imageoutlib)

def create_upc2_barcode(upc, outfile="./upc2.png", resize=1, barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False), imageoutlib=defaultdraw):
    return create_ean2_barcode(upc, outfile, resize, barheight, barwidth, barcolor, hideinfo, imageoutlib)



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: oopfuncs.py - Last Update: 11/15/2024 Ver. 2.12.0 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals, generators, with_statement, nested_scopes
import upcean.validate
import upcean.convert
import upcean.getprefix
import upcean.support
pilsupport = upcean.support.check_for_pil()
pillowsupport = upcean.support.check_for_pillow()
cairosupport = upcean.support.check_for_cairo()
cairosvgsupport = upcean.support.check_for_cairosvg()
svgwritesupport = upcean.support.check_for_svgwrite()
defaultdraw = upcean.support.defaultdraw
if(pilsupport or cairosupport or svgwritesupport):
    import upcean.fonts
    import upcean.xml
    import upcean.encode
if(pilsupport):
    import upcean.decode

''' // Object-oriented classes and functions by Kazuki Przyborowski '''


class encode:
    ''' // Barcode Types '''
    EAN2 = "ean2"
    UPCS2 = "ean2"
    EAN5 = "ean5"
    UPCS5 = "ean5"
    UPCA = "upca"
    GOODWILL = "goodwill"
    UPCE = "upce"
    EAN13 = "ean13"
    EAN8 = "ean8"
    STF = "stf"
    ITF = "itf"
    ITF14 = "itf14"
    CODE11 = "code11"
    CODE39 = "code39"
    CODE93 = "code93"
    CODE128 = "code128"
    CODABAR = "codabar"
    MSI = "msi"
    bctype_dict = upcean.support.bctype_dict
    bctype_dict_alt = upcean.support.bctype_dict_alt
    bctype_list = upcean.support.bctype_list
    bctype_tuple = upcean.support.bctype_tuple
    bctype_name = upcean.support.bctype_name

    def __init__(self, type=None, code=None):
        if(type is not None):
            self.type = type
        if(code is not None):
            self.code = code
        self.filename = "./barcode.png"
        self.inimage = None
        self.size = 1
        self.hidesn = False
        self.hidecd = False
        self.hidetext = False
        self.shiftxy = (0, 0)
        self.barheight = (48, 54)
        self.barwidth = (1, 1)
        self.barcolor = (0, 0, 0)
        self.textcolor = (0, 0, 0)
        self.bgcolor = (255, 255, 255)
        self.imageoutlib = "pillow"
        self.return_check = False
        self.return_type = "dict"
    ''' // support.py funtions '''

    def supported_barcodes(self, return_type=None):
        if(return_type is None):
            return_type = self.return_type
        return upcean.support.supported_barcodes(return_type)

    def barcode_support(self, return_type=None):
        if(return_type is None):
            return_type = self.return_type
        return upcean.support.barcode_support(return_type)

    def get_barcode_name(self, barcode_type=None):
        if(barcode_type is None):
            barcode_type = self.type
        return upcean.support.get_barcode_name(barcode_type)

    def check_for_pil(self):
        return upcean.support.check_for_pil()

    def check_for_pillow(self):
        return upcean.support.check_for_pillow()

    def get_pil_version(self, barcode_type=None):
        if(barcode_type is None):
            barcode_type = self.type
        return upcean.support.get_pil_version()

    def get_pillow_version(self, barcode_type=None):
        if(barcode_type is None):
            barcode_type = self.type
        return upcean.support.get_pillow_version()

    def get_python_info(self, barcode_type=None):
        if(barcode_type is None):
            barcode_type = self.type
        return upcean.support.get_pillow_version()
    ''' // barcodes/__init__.py funtions '''
    if(pilsupport or cairosupport or svgwritesupport):
        def create_barcode(self, filename=None, size=None):
            if(filename is None):
                filename = self.filename
            if(size is None):
                size = self.size
            return upcean.encode.create_barcode(self.type, self.code, filename, size, self.barheight, self.barwidth, (self.barcolor, self.textcolor, self.bgcolor), (self.hidesn, self.hidecd, self.hidetext), self.imageoutlib)

        def validate_create_barcode(self, filename=None, size=None):
            if(filename is None):
                filename = self.filename
            if(size is None):
                size = self.size
            return upcean.encode.validate_create_barcode(self.type, self.code, filename, size, self.barheight, self.barwidth, (self.barcolor, self.textcolor, self.bgcolor), (self.hidesn, self.hidecd, self.hidetext), self.imageoutlib)

        def fix_create_barcode(self, filename=None, size=None):
            if(filename is None):
                filename = self.filename
            if(size is None):
                size = self.size
            return upcean.encode.fix_create_barcode(self.type, self.code, filename, size, self.barheight, self.barwidth, (self.barcolor, self.textcolor, self.bgcolor), (self.hidesn, self.hidecd, self.hidetext), self.imageoutlib)

        def draw_barcode(self, size=None):
            if(size is None):
                size = self.size
            return upcean.encode.draw_barcode(self.type, self.code, size, self.barheight, self.barwidth, (self.barcolor, self.textcolor, self.bgcolor), (self.hidesn, self.hidecd, self.hidetext), self.imageoutlib)

        def validate_draw_barcode(self, size=None):
            if(size is None):
                size = self.size
            return upcean.encode.validate_draw_barcode(self.type, self.code, size, self.barheight, self.barwidth, (self.barcolor, self.textcolor, self.bgcolor), (self.hidesn, self.hidecd, self.hidetext), self.imageoutlib)

        def fix_draw_barcode(self, size=None):
            if(size is None):
                size = self.size
            return upcean.encode.fix_draw_barcode(self.type, self.code, size, self.barheight, self.barwidth, (self.barcolor, self.textcolor, self.bgcolor), (self.hidesn, self.hidecd, self.hidetext), self.imageoutlib)

        def encode_barcode(self, size=None):
            if(size is None):
                size = self.size
            return upcean.encode.encode_barcode(self.type, self.inimage, self.code, size, self.shiftxy, self.barheight, self.barwidth, (self.barcolor, self.textcolor, self.bgcolor), (self.hidesn, self.hidecd, self.hidetext))

        def validate_encode_barcode(self, size=None):
            if(size is None):
                size = self.size
            return upcean.encode.validate_encode_barcode(self.type, self.inimage, self.code, size, self.shiftxy, self.barheight, self.barwidth, (self.barcolor, self.textcolor, self.bgcolor), (self.hidesn, self.hidecd, self.hidetext))

        def fix_encode_barcode(self, size=None):
            if(size is None):
                size = self.size
            return upcean.encode.fix_encode_barcode(self.type, self.inimage, self.code, size, self.shiftxy, self.barheight, self.barwidth, (self.barcolor, self.textcolor, self.bgcolor), (self.hidesn, self.hidecd, self.hidetext))
    ''' // validate/__init__.py funtions '''

    def validate_checksum(self):
        return upcean.validate.validate_checksum(self.type, self.code, self.return_check)

    def validate_luhn_checksum(self):
        return upcean.validate.validate_luhn_checksum(self.code, self.codelen, self.return_check)

    def validate_usps_checksum(self):
        return upcean.validate.validate_usps_checksum(self.code, self.return_check)

    def validate_ups_checksum(self):
        return upcean.validate.validate_ups_checksum(self.code, self.return_check)

    def validate_fedex_checksum(self):
        return upcean.validate.validate_fedex_checksum(self.code, self.return_check)

    def validate_imei_checksum(self):
        return upcean.validate.validate_imei_checksum(self.code, self.return_check)

    def validate_bcn_checksum(self):
        return upcean.validate.validate_bcn_checksum(self.code, self.return_check)

    def get_checksum(self):
        return upcean.validate.get_checksum(self.type, self.code)

    def get_luhn_checksum(self):
        return upcean.validate.get_luhn_checksum(self.code, self.codelen)

    def get_code11_checksum(self):
        return upcean.validate.get_code11_checksum(self.code)

    def get_code93_checksum(self):
        return upcean.validate.get_code93_checksum(self.code)

    def get_code128_checksum(self):
        return upcean.validate.get_code128_checksum(self.code)

    def get_digital_root(self):
        return upcean.validate.get_digital_root(self.number)

    def get_usps_checksum(self):
        return upcean.validate.get_usps_checksum(self.code)

    def get_ups_checksum(self):
        return upcean.validate.get_ups_checksum(self.code)

    def get_fedex_checksum(self):
        return upcean.validate.get_fedex_checksum(self.code)

    def get_imei_checksum(self):
        return upcean.validate.get_imei_checksum(self.code)

    def get_bcn_checksum(self):
        return upcean.validate.get_bcn_checksum(self.code)

    def fix_checksum(self):
        return upcean.validate.fix_checksum(self.type, self.code)

    def fix_luhn_checksum(self):
        return upcean.validate.fix_luhn_checksum(self.code, self.codelen)

    def fix_usps_checksum(self):
        return upcean.validate.fix_usps_checksum(self.code)

    def fix_ups_checksum(self):
        return upcean.validate.fix_ups_checksum(self.code)

    def fix_fedex_checksum(self):
        return upcean.validate.fix_fedex_checksum(self.code)

    def fix_imei_checksum(self):
        return upcean.validate.fix_imei_checksum(self.code)

    def fix_bcn_checksum(self):
        return upcean.validate.fix_bcn_checksum(self.code)
    ''' // convert/__init__.py funtions '''

    def convert_barcode(self, outtype=None):
        if(outtype is None):
            outtype = self.outtype
        return upcean.convert.convert_barcode(self.type, outtype, self.code)

    def print_convert_barcode(self, outtype=None):
        if(outtype is None):
            outtype = self.outtype
        return upcean.convert.print_convert_barcode(self.type, outtype, self.code)

    def make_barcode(self, numbersystem, manufacturer, product):
        if(numbersystem is None):
            numbersystem = self.numbersystem
        if(manufacturer is None):
            manufacturer = self.manufacturer
        if(product is None):
            product = self.product
        return upcean.convert.make_barcode(self.type, self.numbersystem, self.manufacturer, self.product)
    ''' // getsfname.py funtions '''
    if(pilsupport or cairosupport):
        def get_save_filename(self, filename=None):
            if(filename is None):
                filename = self.filename
            return upcean.predraw.get_save_filename(self.filename)
    ''' // getprefix/__init__.py funtions '''

    def get_barcode_info(self):
        return upcean.getprefix.get_barcode_info(self.type, self.code)


barcode = encode

''' // Object-oriented classes and functions by Kazuki Przyborowski '''


class decode:
    ''' // Barcode Types '''
    EAN2 = "ean2"
    UPCS2 = "ean2"
    EAN5 = "ean5"
    UPCS5 = "ean5"
    UPCA = "upca"
    GOODWILL = "goodwill"
    UPCE = "upce"
    EAN13 = "ean13"
    EAN8 = "ean8"
    STF = "stf"
    ITF = "itf"
    ITF14 = "itf14"
    CODE11 = "code11"
    CODE39 = "code39"
    CODE93 = "code93"
    CODE128 = "code128"
    CODABAR = "codabar"
    MSI = "msi"
    bctype_dict = upcean.support.bctype_dict
    bctype_dict_alt = upcean.support.bctype_dict_alt
    bctype_list = upcean.support.bctype_list
    bctype_tuple = upcean.support.bctype_tuple
    bctype_name = upcean.support.bctype_name

    def __init__(self, type=None, filename=None):
        if(type is not None):
            self.type = type
        if(filename is not None):
            self.filename = filename
        self.code = None
        self.info = None
        self.size = 1
        self.barheight = (48, 54)
        self.barwidth = (1, 1)
        self.shiftxy = (0, 0)
        self.barcolor = (0, 0, 0)
        self.textcolor = (0, 0, 0)
        self.bgcolor = (255, 255, 255)
        self.imageoutlib = "pillow"
        self.return_check = False
        self.return_type = "dict"
        self.locatebarcode = False
        self.shiftcheck = False
    if(pilsupport):
        def decode_barcode(self, filename=None, size=None):
            if(filename is None):
                filename = self.filename
            if(size is None):
                size = self.size
            self.code = upcean.decode.decode_barcode(self.type, filename, size, self.barheight, self.barwidth, self.shiftcheck, self.shiftxy, (
                self.barcolor, self.textcolor, self.bgcolor), self.locatebarcode, self.imageoutlib)
            return self.code

        def get_barcode_info(self, filename=None, size=None):
            if(filename is None):
                filename = self.filename
            if(size is None):
                size = self.size
            self.info = upcean.decode.get_barcode_location(self.type, filename, size, self.barheight, self.barwidth,
                                                           self.shiftcheck, self.shiftxy, (self.barcolor, self.textcolor, self.bgcolor), self.imageoutlib)
            return self.info

        def get_barcode_location(self, filename=None, size=None):
            if(filename is None):
                filename = self.filename
            if(size is None):
                size = self.size
            self.info = upcean.decode.get_barcode_location(self.type, filename, size, self.barheight, self.barwidth,
                                                           self.shiftcheck, self.shiftxy, (self.barcolor, self.textcolor, self.bgcolor), self.imageoutlib)
            return self.info

        def validate_decode_barcode(self, filename=None, size=None):
            if(filename is None):
                filename = self.filename
            if(size is None):
                size = self.size
            self.code = upcean.decode.decode_barcode(self.type, filename, size, self.barheight, self.barwidth, self.shiftcheck, self.shiftxy, (
                self.barcolor, self.textcolor, self.bgcolor), self.locatebarcode, self.imageoutlib)
            return self.code

        def fix_decode_barcode(self, filename=None, size=None):
            if(filename is None):
                filename = self.filename
            if(size is None):
                size = self.size
            self.code = upcean.decode.fix_decode_barcode(self.type, filename, size, self.barheight, self.barwidth, self.shiftcheck, self.shiftxy, (
                self.barcolor, self.textcolor, self.bgcolor), self.locatebarcode, self.imageoutlib)
            return self.code



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: __init__.py - Last Update: 11/15/2024 Ver. 2.12.0 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals, generators, with_statement, nested_scopes
import os

implib = False
pkgres = False
try:
    import importlib.resources
    implib = True
except ImportError:
    implib = False
    try:
        import pkg_resources
        pkgres = True
    except ImportError:
        pkgres = False

if(implib):
    fontpathocra = os.path.join(
        importlib.resources.files(__name__), "OCRA.otf")
    fontpathocraalt = os.path.join(
        importlib.resources.files(__name__), "OCRA.ttf")
    fontpathocrb = os.path.join(
        importlib.resources.files(__name__), "OCRB.otf")
    fontpathocrbalt = os.path.join(
        importlib.resources.files(__name__), "OCRB.ttf")
    fontpath = os.path.dirname(fontpathocrb)
elif(pkgres):
    fontpathocra = pkg_resources.resource_filename(__name__, "OCRA.otf")
    fontpathocraalt = pkg_resources.resource_filename(__name__, "OCRA.ttf")
    fontpathocrb = pkg_resources.resource_filename(__name__, "OCRB.otf")
    fontpathocrbalt = pkg_resources.resource_filename(__name__, "OCRB.ttf")
    fontpath = os.path.dirname(fontpathocrb)
elif(not pkgres):
    fontpathocra = os.path.dirname(__file__)+os.sep+"OCRA.otf"
    fontpathocraalt = os.path.dirname(__file__)+os.sep+"OCRA.ttf"
    fontpathocrb = os.path.dirname(__file__)+os.sep+"OCRB.otf"
    fontpathocrbalt = os.path.dirname(__file__)+os.sep+"OCRB.ttf"
    fontpath = os.path.dirname(fontpathocrb)
else:
    fontpathocra = os.path.dirname(__file__)+os.sep+"OCRA.otf"
    fontpathocraalt = os.path.dirname(__file__)+os.sep+"OCRA.ttf"
    fontpathocrb = os.path.dirname(__file__)+os.sep+"OCRB.otf"
    fontpathocrbalt = os.path.dirname(__file__)+os.sep+"OCRB.ttf"
    fontpath = os.path.dirname(fontpathocrb)

os.environ['FONTCONFIG_PATH'] = fontpath



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: goodwill.py - Last Update: 11/15/2024 Ver. 2.12.0 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals, generators, with_statement, nested_scopes
from upcean.predraw import *
import re
import sys
import upcean.support
try:
    file
except NameError:
    from io import IOBase as file
import upcean.getprefix.getprefix
try:
    from io import StringIO, BytesIO
except ImportError:
    try:
        from cStringIO import StringIO
        from cStringIO import StringIO as BytesIO
    except ImportError:
        from StringIO import StringIO
        from StringIO import StringIO as BytesIO
import upcean.encode.ean2
import upcean.encode.ean5
from upcean.encode.upca import *
pilsupport = upcean.support.check_for_pil()
pillowsupport = upcean.support.check_for_pillow()
cairosupport = upcean.support.check_for_cairo()
qahirahsupport = upcean.support.check_for_qahirah()
cairosvgsupport = upcean.support.check_for_cairosvg()
svgwritesupport = upcean.support.check_for_svgwrite()
wandsupport = upcean.support.check_for_wand()
magicksupport = upcean.support.check_for_magick()
defaultdraw = upcean.support.defaultdraw
if(pilsupport or pillowsupport):
    import upcean.predraw.prepil

if(cairosupport):
    import upcean.predraw.precairo
if(svgwritesupport):
    import upcean.predraw.presvgwrite


def create_goodwill_barcode(upc, outfile="./goodwill.png", resize=1, shiftxy=(0, 0), barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False), imageoutlib=defaultdraw):
    upc = str(upc)
    hidesn = hideinfo[0]
    hidecd = hideinfo[1]
    hidetext = hideinfo[2]
    imageoutlib = imageoutlib.lower()
    if(not pilsupport and imageoutlib == "pillow"):
        imageoutlib = "cairo"
    if(not cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
        imageoutlib = "pillow"
    if(not cairosupport and imageoutlib == "cairosvg"):
        imageoutlib = "pillow"
    if(imageoutlib != "pillow" and imageoutlib != "cairo" and imageoutlib != "cairosvg"):
        imageoutlib = "pillow"
    if(not pilsupport and not cairosupport):
        return False
    if(outfile is None):
        if(imageoutlib == "cairosvg"):
            oldoutfile = None
            outfile = None
            outfileext = "SVG"
        else:
            oldoutfile = None
            outfile = None
            outfileext = None
    else:
        oldoutfile = upcean.predraw.get_save_filename(
            outfile, imageoutlib)
        if(isinstance(oldoutfile, tuple) or isinstance(oldoutfile, list)):
            del(outfile)
            outfile = oldoutfile[0]
            outfileext = oldoutfile[1]
            if(cairosupport and imageoutlib == "cairo" and outfileext == "SVG"):
                imageoutlib = "cairosvg"
            if(cairosupport and imageoutlib == "cairosvg" and outfileext != "SVG"):
                imageoutlib = "cairo"
    upc_pieces = None
    supplement = None
    if(re.findall("([0-9]+)([ |\\|]{1})([0-9]{2})$", upc)):
        upc_pieces = re.findall("([0-9]+)([ |\\|]{1})([0-9]{2})$", upc)
        upc_pieces = upc_pieces[0]
        upc = upc_pieces[0]
        supplement = upc_pieces[2]
    if(re.findall("([0-9]+)([ |\\|]){1}([0-9]{5})$", upc)):
        upc_pieces = re.findall("([0-9]+)([ |\\|]){1}([0-9]{5})$", upc)
        upc_pieces = upc_pieces[0]
        upc = upc_pieces[0]
        supplement = upc_pieces[2]
    if(len(upc) > 12 or len(upc) < 12):
        return False
    if(not re.findall("^([0-9]*[\\.]?[0-9])", str(resize)) or int(resize) < 1):
        resize = 1
    if(pilsupport and imageoutlib == "pillow"):
        try:
            pil_ver = Image.PILLOW_VERSION
            pil_ver = pil_ver.split(".")
            pil_ver = [int(x) for x in pil_ver]
            pil_is_pillow = True
        except AttributeError:
            try:
                pil_ver = Image.VERSION
                pil_is_pillow = False
            except AttributeError:
                pil_ver = Image.__version__
                pil_is_pillow = True
            except NameError:
                pil_ver = Image.__version__
                pil_is_pillow = True
            pil_ver = pil_ver.split(".")
            pil_ver = [int(x) for x in pil_ver]
        except NameError:
            try:
                pil_ver = Image.VERSION
                pil_is_pillow = False
            except AttributeError:
                pil_ver = Image.__version__
                pil_is_pillow = True
            except NameError:
                pil_ver = Image.__version__
                pil_is_pillow = True
            pil_ver = pil_ver.split(".")
            pil_ver = [int(x) for x in pil_ver]
        pil_addon_fix = 0
        cairo_addon_fix = 0
        pil_prevercheck = [str(x) for x in pil_ver]
        pil_vercheck = int(
            pil_prevercheck[0]+pil_prevercheck[1]+pil_prevercheck[2])
        if(pil_is_pillow and pil_vercheck >= 210 and pil_vercheck < 220):
            pil_addon_fix = int(resize) * 2
            cairo_addon_fix = 0
    elif(pilsupport and imageoutlib == "pillow"):
        pil_addon_fix = 0
        cairo_addon_fix = 0
    elif(cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
        pil_addon_fix = 0
        cairo_addon_fix = (9 * (int(resize) * barwidth[1]))
    else:
        pil_addon_fix = 0
        cairo_addon_fix = 0
    cairo_addon_fix += (shiftxy[1] * (int(resize) * barwidth[1]))
    upc_matches = re.findall("(\\d{1})(\\d{5})(\\d{5})(\\d{1})", upc)
    if(len(upc_matches) <= 0):
        return False
    goodwillinfo = upcean.getprefix.getprefix.get_goodwill_upca_barcode_info(
        upc)
    barcolor = ((0, 0, 0), (0, 0, 0), (255, 255, 255))
    if(goodwillinfo['tagcolor'] == "Pink"):
        barcolor = ((0, 0, 0), (0, 0, 0), (255, 192, 203))
    elif(goodwillinfo['tagcolor'] == "Yellow"):
        barcolor = ((0, 0, 0), (0, 0, 0), (255, 255, 0))
    elif(goodwillinfo['tagcolor'] == "Green"):
        barcolor = ((0, 0, 0), (0, 0, 0), (207, 240, 236))
    elif(goodwillinfo['tagcolor'] == "Blue"):
        barcolor = ((0, 0, 0), (0, 0, 0), (12, 191, 233))
    elif(goodwillinfo['tagcolor'] == "Orange"):
        barcolor = ((0, 0, 0), (0, 0, 0), (255, 162, 0))
    else:
        barcolor = ((0, 0, 0), (0, 0, 0), (255, 255, 255))
    if(not goodwillinfo):
        return False
    upc_matches = upc_matches[0]
    PrefixDigit = upc_matches[0]
    LeftDigit = list(str(upc_matches[0])+str(upc_matches[1]))
    RightDigit = list(str(upc_matches[2])+str(upc_matches[3]))
    CheckDigit = upc_matches[3]
    upc_size_add = 0
    if(supplement is not None and len(supplement) == 2):
        upc_size_add = 29
    if(supplement is not None and len(supplement) == 5):
        upc_size_add = 56
    if(pilsupport and imageoutlib == "pillow"):
        upc_preimg = Image.new(
            "RGB", (113 + upc_size_add, barheight[1] + (45 * barwidth[0])))
        upc_img = ImageDraw.Draw(upc_preimg)
        upc_img.rectangle(
            [(0, 0), (113 + upc_size_add, barheight[1] + (45 * barwidth[0]))], fill=barcolor[2])
    if(cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
        if(outfileext == "SVG"):
            if(outfile is None):
                imgoutfile = None
            else:
                if(sys.version[0] == "2"):
                    imgoutfile = StringIO()
                if(sys.version[0] >= "3"):
                    imgoutfile = BytesIO()
            upc_preimg = cairo.SVGSurface(
                imgoutfile, (113 + upc_size_add, barheight[1] + (45 * barwidth[0])))
        elif(outfileext == "PDF"):
            upc_preimg = cairo.PDFSurface(
                None, ((113 * barwidth[0]) + upc_size_add) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize))
        elif(outfileext == "PS" or outfileext == "EPS"):
            upc_preimg = cairo.PSSurface(
                None, ((113 * barwidth[0]) + upc_size_add) * int(resize), (barheightadd + (9 * barwidth[1])) * int(resize))
            if(outfileext == "EPS"):
                upc_preimg.set_eps(True)
            else:
                upc_preimg.set_eps(False)
        else:
            upc_preimg = cairo.ImageSurface(
                cairo.FORMAT_RGB24, (113 + upc_size_add, barheight[1] + (45 * barwidth[0])))
        upc_img = cairo.Context(upc_preimg)
        upc_img.set_antialias(cairo.ANTIALIAS_NONE)
        upc_img.rectangle(0, 0, 113 + upc_size_add,
                          barheight[1] + (45 * barwidth[0]))
        upc_img.set_source_rgb(barcolor[2][0], barcolor[2][1], barcolor[2][2])
        upc_img.fill()
    drawColorText(upc_img, 12 * int(resize * barwidth[0]), 30 + (23 * (int(resize) - 1)) - (4 * (int(
        resize * barwidth[0]) - 1)), (4 * int(resize * barwidth[0])), "Goodwill", barcolor[1], "ocrb", imageoutlib)
    if(len(goodwillinfo['pricewdnz']) < 4):
        goodwillinfo['pricewdnz'] = "0"+goodwillinfo['pricewdnz']
    upc_size_add = 0
    if(len(goodwillinfo['pricewdnz']) == 5):
        upc_size_add = -14
    if(len(goodwillinfo['pricewdnz']) == 6):
        upc_size_add = -30
    drawColorText(upc_img, 16 * int(resize * barwidth[0]), 36 + upc_size_add + (23 * (int(resize) - 1)) - (4 * (int(
        resize * barwidth[0]) - 1)), (75 * int(resize * barwidth[0])), "$"+goodwillinfo['pricewdnz'], barcolor[1], "ocrb", imageoutlib)
    if(pilsupport and imageoutlib == "pillow"):
        if(supplement is not None and len(supplement) == 2):
            upc_sup_img = upcean.encode.ean2.draw_ean2sup_barcode(
                supplement, resize, shiftxy, barheight, barwidth, barcolor, hideinfo, imageoutlib)
            if(upc_sup_img):
                new_upc_img.paste(upc_sup_img, (113 * int(resize), 0))
                del(upc_sup_img)
        if(supplement is not None and len(supplement) == 5):
            upc_sup_img = upcean.encode.ean5.draw_ean5sup_barcode(
                supplement, resize, shiftxy, barheight, barwidth, barcolor, hideinfo, imageoutlib)
            if(upc_sup_img):
                new_upc_img.paste(upc_sup_img, (113 * int(resize), 0))
                del(upc_sup_img)
    if(cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
        if(supplement != None and len(supplement) == 2):
            upc_sup_img = draw_ean2sup_barcode(
                supplement, 1, hideinfo, barheight, barwidth, barcolor, imageoutlib)
            new_upc_img.set_source_surface(upc_sup_img, 113, 0)
            new_upc_img.paint()
            del(upc_sup_img)
        if(supplement != None and len(supplement) == 5):
            upc_sup_img = draw_ean5sup_barcode(
                supplement, 1, hideinfo, barheight, barwidth, barcolor, imageoutlib)
            new_upc_img.set_source_surface(upc_sup_img, 113, 0)
            new_upc_img.paint()
            del(upc_sup_img)
    exargdict = {'comment': "upca; goodwill; "+upc}
    if(oldoutfile is None or isinstance(oldoutfile, bool)):
        if(pilsupport and imageoutlib == "pillow"):
            return [upc_img, upc_preimg, {'upc': upc, 'outfile': outfile, 'resize': resize, 'shiftxy': shiftxy, 'barheight': barheight, 'barwidth': barwidth, 'barcolor': barcolor, 'hideinfo': hideinfo, 'imageoutlib': imageoutlib}, upc_array]
        if(cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
            return [upc_img, upc_preimg, {'upc': upc, 'outfile': outfile, 'resize': resize, 'shiftxy': shiftxy, 'barheight': barheight, 'barwidth': barwidth, 'barcolor': barcolor, 'hideinfo': hideinfo, 'imageoutlib': imageoutlib}, upc_array]
    if(sys.version[0] == "2"):
        if(outfile == "-" or outfile == "" or outfile == " " or outfile is None):
            stdoutfile = StringIO()
            if(outfileext == "WEBP"):
                exargdict.update(
                    {'lossless': True, 'quality': 100, 'method': 6})
            elif(outfileext == "JPEG"):
                exargdict.update(
                    {'quality': 100, 'optimize': True, 'progressive': True})
            elif(outfileext == "PNG"):
                exargdict.update({'optimize': True, 'compress_level': 9})
            else:
                exargdict = {'comment': "upca; goodwill; "+upc}
            try:
                if(pilsupport and imageoutlib == "pillow"):
                    if(outfileext == "BYTES"):
                        stdoutfile.write(upc_preimg.tobytes())
                        stdoutfile.seek(0)
                        return stdoutfile
                    elif(outfileext == "XBM"):
                        stdoutfile.write(
                            upc_preimg.convert(mode="1").tobitmap())
                        stdoutfile.seek(0)
                        return stdoutfile
                    elif(outfileext == "XPM"):
                        upc_preimg.convert(mode="P").save(
                            stdoutfile, outfileext, **exargdict)
                        stdoutfile.seek(0)
                        return stdoutfile
                    else:
                        upc_preimg.save(stdoutfile, outfileext, **exargdict)
                        stdoutfile.seek(0)
                        return stdoutfile
                if(cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
                    if(outfileext == "BYTES"):
                        stdoutfile.write(upc_preimg.get_data().tobytes())
                        stdoutfile.seek(0)
                        return stdoutfile
                    elif(outfileext == "SVG" or outfileext == "PDF" or outfileext == "PS" or outfileext == "EPS" or imageoutlib == "cairosvg"):
                        upc_preimg.flush()
                        upc_preimg.finish()
                        imgoutfile.seek(0)
                        svgouttext = imgoutfile.read()
                        stdoutfile.write(svgouttext)
                        imgoutfile.close()
                        stdoutfile.seek(0)
                        return stdoutfile
                    else:
                        upc_preimg.write_to_png(stdoutfile)
                        stdoutfile.seek(0)
                        return stdoutfile
            except:
                return False
    if(sys.version[0] >= "3"):
        stdoutfile = BytesIO()
        if(outfile == "-" or outfile == "" or outfile == " " or outfile is None):
            if(outfileext == "WEBP"):
                exargdict.update(
                    {'lossless': True, 'quality': 100, 'method': 6})
            elif(outfileext == "JPEG"):
                exargdict.update(
                    {'quality': 100, 'optimize': True, 'progressive': True})
            elif(outfileext == "PNG"):
                exargdict.update({'optimize': True, 'compress_level': 9})
            else:
                exargdict = {'comment': upc}
            try:
                if(pilsupport and imageoutlib == "pillow"):
                    if(outfileext == "BYTES"):
                        stdoutfile.write(upc_preimg.tobytes())
                        stdoutfile.seek(0)
                        return stdoutfile
                    elif(outfileext == "XBM"):
                        stdoutfile.write(
                            upc_preimg.convert(mode='1').tobitmap())
                        stdoutfile.seek(0)
                        return stdoutfile
                    elif(outfileext == "XPM"):
                        upc_preimg.convert(mode="P").save(
                            stdoutfile, outfileext, **exargdict)
                        stdoutfile.seek(0)
                        return stdoutfile
                    else:
                        upc_preimg.save(stdoutfile, outfileext, **exargdict)
                        stdoutfile.seek(0)
                        return stdoutfile
                if(cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
                    if(outfileext == "BYTES"):
                        stdoutfile.write(upc_preimg.get_data().tobytes())
                        stdoutfile.seek(0)
                        return stdoutfile
                    elif(outfileext == "SVG" or outfileext == "PDF" or outfileext == "PS" or outfileext == "EPS" or imageoutlib == "cairosvg"):
                        upc_preimg.flush()
                        upc_preimg.finish()
                        imgoutfile.seek(0)
                        svgouttext = imgoutfile.read()
                        stdoutfile.write(svgouttext)
                        imgoutfile.close()
                        stdoutfile.seek(0)
                        return stdoutfile
                    else:
                        upc_preimg.write_to_png(stdoutfile)
                        stdoutfile.seek(0)
                        return stdoutfile
            except:
                return False
    if(outfile != "-" and outfile != "" and outfile != " "):
        if(outfileext == "WEBP"):
            exargdict.update({'lossless': True, 'quality': 100, 'method': 6})
        if(outfileext == "HEIC"):
            exargdict.update({'lossless': True, 'quality': 100})
        elif(outfileext == "JPEG"):
            exargdict.update(
                {'quality': 100, 'optimize': True, 'progressive': True})
        elif(outfileext == "GIF"):
            exargdict.update(
                {'optimize': True})
        elif(outfileext == "PNG"):
            exargdict.update({'optimize': True, 'compress_level': 9, 'quality': 100})
            if(pilsupport):
                # Add a comment to the image
                info = PngImagePlugin.PngInfo()
                info.add_text("Comment", upc)
                exargdict.update({'pnginfo': info})
        else:
            exargdict = {'comment': upc}
        try:
            if(pilsupport and imageoutlib == "pillow"):
                if(outfileext == "BYTES"):
                    with open(outfile, 'wb+') as f:
                        f.write(upc_preimg.tobytes())
                elif(outfileext == "XBM"):
                    with open(outfile, 'wb+') as f:
                        f.write(upc_preimg.get_data().tobytes())
                elif(outfileext == "XPM"):
                    upc_preimg.convert(mode="P").save(
                        outfile, outfileext, **exargdict)
                else:
                    upc_preimg.save(outfile, outfileext, **exargdict)
            if(cairosupport and (imageoutlib == "cairo" or imageoutlib == "cairosvg")):
                if(outfileext == "BYTES"):
                    with open(outfile, 'wb+') as f:
                        f.write(upc_preimg.get_data().tobytes())
                    return True
                elif(outfileext == "SVG" or outfileext == "PDF" or outfileext == "PS" or outfileext == "EPS" or imageoutlib == "cairosvg"):
                    upc_preimg.flush()
                    upc_preimg.finish()
                    imgoutfile.seek(0)
                    svgouttext = imgoutfile.read()
                    with open(outfile, 'wb+') as f:
                        f.write(svgouttext)
                    return True
                else:
                    upc_preimg.write_to_png(outfile)
                    return True
        except Exception:
            return False
    return True


def draw_goodwill_barcode(upc, resize=1, shiftxy=(0, 0), barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False), imageoutlib=defaultdraw):
    return create_goodwill_barcode(upc, None, resize, shiftxy, barheight, barwidth, barcolor, hideinfo, imageoutlib)


def encode_goodwill_barcode(upc, resize=1, shiftxy=(0, 0), barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), hideinfo=(False, False, False), imageoutlib=defaultdraw):
    return create_goodwill_barcode(upc, None, resize, shiftxy, barheight, barwidth, barcolor, hideinfo, imageoutlib)



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: validate.py - Last Update: 11/15/2024 Ver. 2.12.0 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals, generators, with_statement, nested_scopes
import re
import string

'''
// Digital Root
// Source: http://en.wikipedia.org/wiki/Digital_root
'''


def get_digital_root(number):
    number = str(number)
    while(len(str(number)) > 1):
        subnum = list(str(number))
        PreCount = 0
        number = 0
        while (PreCount <= len(subnum)-1):
            number += int(subnum[PreCount])
            PreCount += 1
    return int(number)


def digital_root(number):
    # Special case for when number is 0, as 0 % 9 would also be 0
    if number == 0:
        return 0
    else:
        return number % 9 if number % 9 != 0 else 9


'''
// Luhn Algorithm ( Luhn Formula )
// http://en.wikipedia.org/wiki/Luhn_algorithm#Implementation_of_standard_Mod_10
'''

def validate_luhn_checksum(upc, upclen, return_check=False):
    upc = str(upc)
    upclen = int(upclen)
    upclendwn = upclen - 1

    # Trim the UPC to the specified length if it's too long
    if len(upc) > upclen:
        upc = re.findall("^\\d{" + str(upclen) + "}", upc)[0]

    # If UPC length is incorrect after trimming, return False
    if len(upc) > upclen or len(upc) < upclendwn:
        return False

    # Convert UPC to a list of integers
    upc_digits = [int(digit) for digit in upc]

    # Separate into odd and even positioned sums based on length
    odd_sum = sum(upc_digits[0::2])
    even_sum = sum(upc_digits[1::2])

    # Adjust sums based on parity of upclen
    if upclen % 2 == 0:
        total_sum = (odd_sum * 3) + even_sum
    else:
        total_sum = odd_sum + (even_sum * 3)

    # Calculate checksum digit
    checksum = (10 - (total_sum % 10)) % 10

    # Validate checksum or return it based on the `return_check` flag
    if return_check:
        return str(checksum)

    # If the provided UPC length matches upclen, validate the checksum
    return checksum == upc_digits[-1] if len(upc) == upclen else str(checksum)


def get_luhn_checksum(upc, upclen):
    upc = str(upc)
    upclen = int(upclen)
    return validate_luhn_checksum(upc, upclen, True)


def fix_luhn_checksum(upc, upclen):
    upc = str(upc)
    upclen = int(upclen)
    if(len(upc) > upclen):
        fix_matches = re.findall("^(\\d{"+str(upclen)+"})", upc)
        upc = fix_matches[0]
    return upc+str(get_luhn_checksum(upc, upclen))


# Define a custom character set for mod N (e.g., alphanumeric)
# Here we use base 36 (0-9, A-Z) as an example.
CHARSET = string.digits + string.ascii_uppercase
BASE = len(CHARSET)

# Mapping character to index for quick lookup
CHAR_TO_INDEX = {ch: idx for idx, ch in enumerate(CHARSET)}

def validate_luhn_mod_n_checksum(code, length, return_check=False):
    code = str(code).upper()  # Convert to uppercase to match CHARSET
    length = int(length)
    len_minus_one = length - 1

    # Trim the code to the specified length if it's too long
    if len(code) > length:
        code = re.findall("^[" + CHARSET + "]{" + str(length) + "}", code)[0]

    # If code length is incorrect after trimming, return False
    if len(code) > length or len(code) < len_minus_one:
        return False

    # Convert code to a list of indices in CHARSET
    code_indices = [CHAR_TO_INDEX[char] for char in code]

    # Separate into odd and even positioned sums based on length
    odd_sum = sum(code_indices[0::2])
    even_sum = sum(code_indices[1::2])

    # Adjust sums based on parity of length
    if length % 2 == 0:
        total_sum = (odd_sum * 2) + even_sum
    else:
        total_sum = odd_sum + (even_sum * 2)

    # Calculate checksum index
    checksum_index = (BASE - (total_sum % BASE)) % BASE
    checksum_char = CHARSET[checksum_index]

    # Validate checksum or return it based on the `return_check` flag
    if return_check:
        return checksum_char

    # If the provided code length matches length, validate the checksum
    return checksum_char == code[-1] if len(code) == length else checksum_char


def get_luhn_mod_n_checksum(code, length):
    return validate_luhn_mod_n_checksum(code, length, True)


def fix_luhn_mod_n_checksum(code, length):
    code = str(code).upper()  # Ensure uppercase to match CHARSET
    length = int(length)

    # Trim or pad the code to be one less than the specified length
    if len(code) >= length:
        code = code[:length - 1]
    elif len(code) < length - 1:
        code = code.zfill(length - 1)

    # Calculate and append the correct checksum character
    return code + get_luhn_mod_n_checksum(code, length)



# Tables needed for Verhoeff algorithm
d_table = [
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
    [1, 2, 3, 4, 0, 6, 7, 8, 9, 5],
    [2, 3, 4, 0, 1, 7, 8, 9, 5, 6],
    [3, 4, 0, 1, 2, 8, 9, 5, 6, 7],
    [4, 0, 1, 2, 3, 9, 5, 6, 7, 8],
    [5, 9, 8, 7, 6, 0, 4, 3, 2, 1],
    [6, 5, 9, 8, 7, 1, 0, 4, 3, 2],
    [7, 6, 5, 9, 8, 2, 1, 0, 4, 3],
    [8, 7, 6, 5, 9, 3, 2, 1, 0, 4],
    [9, 8, 7, 6, 5, 4, 3, 2, 1, 0]
]

p_table = [
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
    [1, 5, 7, 6, 2, 8, 3, 0, 9, 4],
    [5, 8, 0, 3, 7, 9, 6, 1, 4, 2],
    [8, 9, 1, 6, 0, 4, 3, 5, 2, 7],
    [9, 4, 5, 3, 1, 2, 6, 8, 7, 0],
    [4, 2, 8, 6, 5, 7, 3, 9, 0, 1],
    [2, 7, 9, 3, 8, 0, 6, 4, 1, 5],
    [7, 0, 4, 6, 9, 1, 3, 2, 5, 8]
]

inv_table = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

# https://en.wikipedia.org/wiki/Verhoeff_algorithm

def validate_verhoeff_checksum(number, length, return_check=False):
    number = str(number)
    length = int(length)

    # Trim the number to the specified length if it's too long
    if len(number) > length:
        number = re.findall("^\\d{" + str(length) + "}", number)[0]

    # If number length is incorrect after trimming, return False
    if len(number) != length:
        return False

    # Calculate Verhoeff checksum
    c = 0
    for i, digit in enumerate(reversed(number)):
        c = d_table[c][p_table[i % 8][int(digit)]]

    checksum = inv_table[c]

    if return_check:
        return str(checksum)

    # Validate checksum by checking if last digit matches
    return checksum == int(number[-1])

def get_verhoeff_checksum(number, length):
    return validate_verhoeff_checksum(number, length, True)

def fix_verhoeff_checksum(number, length):
    number = str(number)
    length = int(length)

    # Ensure number is of length-1 and get checksum to append
    if len(number) >= length:
        number = number[:length - 1]
    elif len(number) < length - 1:
        number = number.zfill(length - 1)

    return number + get_verhoeff_checksum(number, length)

# https://en.wikipedia.org/wiki/Damm_algorithm

# Damm algorithm table (quasigroup)
damm_table = [
    [0, 3, 1, 7, 5, 9, 8, 6, 4, 2],
    [7, 0, 9, 2, 1, 5, 4, 8, 6, 3],
    [4, 2, 0, 6, 8, 7, 1, 3, 5, 9],
    [1, 7, 5, 0, 9, 8, 3, 4, 2, 6],
    [6, 1, 2, 3, 0, 4, 5, 9, 7, 8],
    [3, 6, 7, 4, 2, 0, 9, 5, 8, 1],
    [5, 8, 6, 9, 7, 2, 0, 1, 3, 4],
    [8, 9, 4, 5, 3, 6, 2, 0, 1, 7],
    [9, 4, 3, 8, 6, 1, 7, 2, 0, 5],
    [2, 5, 8, 1, 4, 3, 6, 7, 9, 0]
]

def validate_damm_checksum(number, length, return_check=False):
    number = str(number)
    length = int(length)

    # Trim the number to the specified length if it's too long
    if len(number) > length:
        number = re.findall("^\\d{" + str(length) + "}", number)[0]

    # If number length is incorrect after trimming, return False
    if len(number) != length:
        return False

    # Calculate Damm checksum
    interim = 0
    for digit in number[:-1]:
        interim = damm_table[interim][int(digit)]

    checksum = damm_table[interim][int(number[-1])]

    if return_check:
        return str(interim)

    # Validate by checking if the checksum result is 0
    return checksum == 0

def get_damm_checksum(number, length):
    return validate_damm_checksum(number, length, True)

def fix_damm_checksum(number, length):
    number = str(number)
    length = int(length)

    # Ensure number is of length-1 and get checksum to append
    if len(number) >= length:
        number = number[:length - 1]
    elif len(number) < length - 1:
        number = number.zfill(length - 1)

    return number + get_damm_checksum(number, length)


def validate_upca_checksum(upc, return_check=False):
    upc = str(upc)
    
    # Trim UPC to 12 digits if it's too long
    if len(upc) > 12:
        upc = re.findall("^\\d{12}", upc)[0]
    
    # Check for valid length (should be 11 or 12)
    if len(upc) not in {11, 12}:
        return False

    # Convert UPC string to a list of integers
    upc_digits = [int(digit) for digit in upc]
    
    # Calculate the sum of odd and even positions
    odd_sum = sum(upc_digits[0::2]) * 3
    even_sum = sum(upc_digits[1::2][:-1] if len(upc) == 12 else upc_digits[1::2])
    
    # Calculate the checksum digit
    total_sum = odd_sum + even_sum
    checksum = (10 - (total_sum % 10)) % 10
    
    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # If UPC is 12 digits, verify checksum
    return checksum == upc_digits[-1] if len(upc) == 12 else str(checksum)


def get_upca_checksum(upc):
    upc = str(upc)
    return validate_upca_checksum(upc, True)


def fix_upca_checksum(upc):
    upc = str(upc)
    if(len(upc) > 11):
        fix_matches = re.findall("^(\\d{11})", upc)
        upc = fix_matches[0]
    return upc+str(get_upca_checksum(upc))


def validate_upca_alt_checksum(upc, return_check=False):
    upc = str(upc)
    return validate_luhn_checksum(upc, 12, return_check)


def get_upca_alt_checksum(upc):
    upc = str(upc)
    return validate_upca_alt_checksum(upc, True)


def fix_upca_alt_checksum(upc):
    upc = str(upc)
    if(len(upc) > 11):
        fix_matches = re.findall("^(\\d{11})", upc)
        upc = fix_matches[0]
    return upc+str(get_upca_alt_checksum(upc))


def validate_ean13_checksum(upc, return_check=False):
    upc = str(upc)
    
    # Trim UPC to 13 digits if it's too long
    if len(upc) > 13:
        upc = re.findall(r"^\d{13}", upc)[0]
    
    # Check for valid length (should be 12 or 13)
    if len(upc) not in {12, 13}:
        return False

    # Convert UPC string to a list of integers (including checksum digit if present)
    upc_digits = [int(digit) for digit in upc]
    
    # Determine whether to exclude the checksum digit in calculations
    if len(upc) == 13:
        data_digits = upc_digits[:-1]  # Exclude checksum digit
    else:
        data_digits = upc_digits

    # Calculate the sum of odd and even positions
    odd_sum = sum(data_digits[::2])
    even_sum = sum(data_digits[1::2]) * 3

    # Calculate the checksum digit
    total_sum = odd_sum + even_sum
    checksum = (10 - (total_sum % 10)) % 10

    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # If UPC is 13 digits, verify checksum using upc_digits[-1]
    return checksum == upc_digits[-1] if len(upc) == 13 else str(checksum)


def get_ean13_checksum(upc):
    upc = str(upc)
    return validate_ean13_checksum(upc, True)


def fix_ean13_checksum(upc):
    upc = str(upc)
    if(len(upc) > 12):
        fix_matches = re.findall("^(\\d{12})", upc)
        upc = fix_matches[0]
    return upc+str(get_ean13_checksum(upc))


def validate_ean13_alt_checksum(upc, return_check=False):
    upc = str(upc)
    return validate_luhn_checksum(upc, 13, return_check)


def get_ean13_alt_checksum(upc):
    upc = str(upc)
    return validate_ean13_alt_checksum(upc, True)


def fix_ean13_alt_checksum(upc):
    upc = str(upc)
    if(len(upc) > 12):
        fix_matches = re.findall("^(\\d{12})", upc)
        upc = fix_matches[0]
    return upc+str(get_ean13_alt_checksum(upc))


def validate_itf6_checksum(upc, return_check=False):
    upc = str(upc)
    
    # Trim UPC to 6 digits if it's too long
    if len(upc) > 6:
        upc = re.findall("^\\d{6}", upc)[0]
    
    # Check for valid length (should be 5 or 6)
    if len(upc) not in {5, 6}:
        return False

    # Convert UPC string to a list of integers
    upc_digits = [int(digit) for digit in upc]
    
    # Calculate the checksum digit
    total_sum = (3 * upc_digits[0] + upc_digits[1] + 3 * upc_digits[2] + 
                 upc_digits[3] + 3 * upc_digits[4])
    checksum = (10 - (total_sum % 10)) % 10
    
    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # If UPC is 6 digits, verify checksum
    return checksum == upc_digits[-1] if len(upc) == 6 else str(checksum)


def get_itf6_checksum(upc):
    upc = str(upc)
    return validate_itf6_checksum(upc, True)


def fix_itf6_checksum(upc):
    upc = str(upc)
    if(len(upc) > 5):
        fix_matches = re.findall("^(\\d{5})", upc)
        upc = fix_matches[0]
    return upc+str(get_itf6_checksum(upc))


def validate_itf6_alt_checksum(upc, return_check=False):
    upc = str(upc)
    return validate_luhn_checksum(upc, 6, return_check)


def get_itf6_checksum(upc):
    upc = str(upc)
    return validate_itf6_alt_checksum(upc, True)


def fix_itf6_alt_checksum(upc):
    upc = str(upc)
    if(len(upc) > 5):
        fix_matches = re.findall("^(\\d{5})", upc)
        upc = fix_matches[0]
    return upc+str(get_itf6_alt_checksum(upc))


def validate_itf14_checksum(upc, return_check=False):
    upc = str(upc)
    
    # Trim UPC to 14 digits if it's too long
    if len(upc) > 14:
        upc = re.findall("^\\d{14}", upc)[0]
    
    # Check for valid length (should be 13 or 14)
    if len(upc) not in {13, 14}:
        return False

    # Convert UPC string to a list of integers (including checksum digit if present)
    upc_digits = [int(digit) for digit in upc]
    
    # Separate data digits (exclude checksum digit if present)
    if len(upc) == 14:
        data_digits = upc_digits[:-1]  # Exclude checksum digit
    else:
        data_digits = upc_digits       # Use all digits if checksum is not present

    # Calculate the sum of odd and even positioned digits
    # For ITF-14:
    # - Odd positions (1st, 3rd, ...) are multiplied by 3
    # - Even positions (2nd, 4th, ...) are added as is
    odd_sum = sum(data_digits[::2]) * 3       # Multiply sum of digits in odd positions by 3
    even_sum = sum(data_digits[1::2])         # Sum of digits in even positions

    # Calculate the checksum digit
    total_sum = odd_sum + even_sum
    checksum = (10 - (total_sum % 10)) % 10

    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # If UPC is 14 digits, verify checksum using upc_digits[-1]
    return checksum == upc_digits[-1] if len(upc) == 14 else str(checksum)


def get_itf14_checksum(upc):
    upc = str(upc)
    return validate_itf14_checksum(upc, True)


def fix_itf14_checksum(upc):
    upc = str(upc)
    if(len(upc) > 13):
        fix_matches = re.findall("^(\\d{13})", upc)
        upc = fix_matches[0]
    return upc+str(get_itf14_checksum(upc))


def validate_itf14_alt_checksum(upc, return_check=False):
    upc = str(upc)
    return validate_luhn_checksum(upc, 14, return_check)


def get_itf14_checksum(upc):
    upc = str(upc)
    return validate_itf14_alt_checksum(upc, True)


def fix_itf14_alt_checksum(upc):
    upc = str(upc)
    if(len(upc) > 13):
        fix_matches = re.findall("^(\\d{13})", upc)
        upc = fix_matches[0]
    return upc+str(get_itf14_alt_checksum(upc))


def validate_itf_checksum(upc):
    upc = str(upc)
    
    # Trim UPC to 14 digits if it's too long
    if len(upc) > 14:
        upc = re.findall("^\\d{14}", upc)[0]
    
    # Check for valid length (should be 13 or 14)
    if len(upc) not in {13, 14}:
        return False

    # Convert UPC string to a list of integers
    upc_digits = [int(digit) for digit in upc]
    
    # Calculate the sum of odd and even positioned digits
    odd_sum = sum(upc_digits[0::2]) * 3
    even_sum = sum(upc_digits[1::2])
    
    # Calculate the checksum digit
    total_sum = odd_sum + even_sum
    checksum = (10 - (total_sum % 10)) % 10
    
    # Return the checksum as a string
    return str(checksum)


def get_itf_checksum(upc):
    upc = str(upc)
    return validate_itf_checksum(upc, True)


def fix_itf_checksum(upc):
    upc = str(upc)
    if(len(upc) > 13):
        fix_matches = re.findall("^(\\d{13})", upc)
        upc = fix_matches[0]
    return upc+str(get_itf_checksum(upc))


def validate_itf_alt_checksum(upc, return_check=False):
    upc = str(upc)
    return validate_luhn_checksum(upc, 14, return_check)


def get_itf_alt_checksum(upc):
    upc = str(upc)
    return validate_itf_alt_checksum(upc, True)


def fix_itf_alt_checksum(upc):
    upc = str(upc)
    if(len(upc) > 13):
        fix_matches = re.findall("^(\\d{13})", upc)
        upc = fix_matches[0]
    return upc+str(get_itf_alt_checksum(upc))


def validate_stf_checksum(upc):
    CheckSum = validate_itf_checksum(upc)
    return str(CheckSum)


def get_stf_checksum(upc):
    CheckSum = get_itf_checksum(upc)
    return str(CheckSum)


def fix_stf_checksum(upc):
    CheckSum = fix_itf_checksum(upc)
    return str(CheckSum)


def validate_ean8_checksum(upc, return_check=False):
    upc = str(upc)
    
    # Trim UPC to 8 digits if it's too long
    if len(upc) > 8:
        upc = re.findall("^\\d{8}", upc)[0]
    
    # Check for valid length (should be 7 or 8)
    if len(upc) not in {7, 8}:
        return False

    # Convert UPC string to a list of integers
    upc_digits = [int(digit) for digit in upc]
    
    # Calculate the sum of odd and even positioned digits
    odd_sum = sum(upc_digits[0::2]) * 3
    even_sum = sum(upc_digits[1::2][:-1] if len(upc) == 8 else upc_digits[1::2])
    
    # Calculate the checksum digit
    total_sum = odd_sum + even_sum
    checksum = (10 - (total_sum % 10)) % 10
    
    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # If UPC is 8 digits, verify checksum
    return checksum == upc_digits[-1] if len(upc) == 8 else str(checksum)


def get_ean8_checksum(upc):
    upc = str(upc)
    return validate_ean8_checksum(upc, True)


def fix_ean8_checksum(upc):
    upc = str(upc)
    if(len(upc) > 7):
        fix_matches = re.findall("^(\\d{7})", upc)
        upc = fix_matches[0]
    return upc+str(get_ean8_checksum(upc))


def validate_upce_checksum(upc, return_check=False):
    upc = str(upc)
    
    # Trim UPC to 8 digits if it's too long
    if len(upc) > 8:
        upc = re.findall("^\\d{8}", upc)[0]
    
    # Validate length and check for number system 0 or 1
    if len(upc) not in {7, 8} or not re.match("^[01]", upc):
        return False

    # Extract CheckDigit if upc is 8 digits
    check_digit = int(upc[-1]) if len(upc) == 8 else None
    upc_digits = [int(digit) for digit in upc[:7]]
    
    # Calculate OddSum and EvenSum based on last digit (upc_digits[6])
    last_digit = upc_digits[6]
    if last_digit == 0:
        odd_sum = sum(upc_digits[i] for i in [0, 2, 3, 5]) * 3
        even_sum = upc_digits[1] + upc_digits[4]
    elif last_digit == 1:
        odd_sum = sum(upc_digits[i] for i in [0, 2, 3, 5]) * 3
        even_sum = upc_digits[1] + 1 + upc_digits[4]
    elif last_digit == 2:
        odd_sum = sum(upc_digits[i] for i in [0, 2, 3, 5]) * 3
        even_sum = upc_digits[1] + 2 + upc_digits[4]
    elif last_digit == 3:
        odd_sum = sum(upc_digits[i] for i in [0, 2, 5]) * 3
        even_sum = upc_digits[1] + upc_digits[3] + upc_digits[4]
    elif last_digit == 4:
        odd_sum = sum(upc_digits[i] for i in [0, 2, 4, 5]) * 3
        even_sum = upc_digits[1] + upc_digits[3]
    elif last_digit in {5, 6, 7, 8, 9}:
        odd_sum = sum(upc_digits[i] for i in [0, 2, 4, 6]) * 3
        even_sum = upc_digits[1] + upc_digits[3] + upc_digits[5]
    
    # Calculate CheckSum
    total_sum = odd_sum + even_sum
    calculated_checksum = (10 - (total_sum % 10)) % 10
    
    # Return the calculated checksum if requested
    if return_check:
        return str(calculated_checksum)
    
    # Validate the checksum if upc is 8 digits
    return calculated_checksum == check_digit if len(upc) == 8 else str(calculated_checksum)


def get_upce_checksum(upc):
    upc = str(upc)
    return validate_upce_checksum(upc, True)


def fix_upce_checksum(upc):
    upc = str(upc)
    if(len(upc) > 7):
        fix_matches = re.findall("^(\\d{7})", upc)
        upc = fix_matches[0]
    return upc+str(get_upce_checksum(upc))


def validate_ean2_checksum(upc, return_check=False):
    upc = str(upc)
    
    # Trim UPC to 2 or 3 digits if it's too long
    if len(upc) > 3:
        upc = re.findall("^\\d{3}", upc)[0]
    
    # Check for valid length (should be 2 or 3)
    if len(upc) not in {2, 3}:
        return False

    # Convert UPC string to a list of integers
    upc_digits = [int(digit) for digit in upc[:2]]
    
    # Calculate the checksum as the first digit modulo 4
    checksum = upc_digits[0] % 4
    
    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # If UPC is 3 digits, validate the checksum
    return checksum == int(upc[2]) if len(upc) == 3 else str(checksum)


def get_ean2_checksum(upc):
    upc = str(upc)
    return validate_ean2_checksum(upc, True)


def fix_ean2_checksum(upc):
    upc = str(upc)
    if(len(upc) > 2):
        fix_matches = re.findall("^(\\d{2})", upc)
        upc = fix_matches[0]
    return upc+str(get_ean2_checksum(upc))


def validate_ean5_checksum(upc, return_check=False):
    upc = str(upc)
    
    # Trim UPC to 6 digits if it's too long
    if len(upc) > 6:
        upc = re.findall("^\\d{6}", upc)[0]
    
    # Check for valid length (should be 5 or 6)
    if len(upc) not in {5, 6}:
        return False

    # Extract digits and calculate checksum
    upc_digits = [int(digit) for digit in upc[:5]]
    checksum = (upc_digits[0] * 3 + upc_digits[1] * 9 + 
                upc_digits[2] * 3 + upc_digits[3] * 9 + 
                upc_digits[4] * 3) % 10

    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # If UPC is 6 digits, validate checksum
    return checksum == int(upc[5]) if len(upc) == 6 else str(checksum)


def get_ean5_checksum(upc):
    upc = str(upc)
    return validate_ean5_checksum(upc, True)


def fix_ean5_checksum(upc):
    upc = str(upc)
    if(len(upc) > 5):
        fix_matches = re.findall("^(\\d{5})", upc)
        upc = fix_matches[0]
    return upc+str(get_ean5_checksum(upc))


'''
// Get USPS Checkdigit by MACY8167
// Source: http://www.mrexcel.com/forum/excel-questions/530675-usps-mod-10-check-digit.html
'''


def validate_usps_checksum(upc, return_check=False):
    upc = str(upc)
    
    # Trim UPC to 22 digits if it's too long
    if len(upc) > 22:
        upc = re.findall("^\\d{22}", upc)[0]
    
    # Check for valid length (should be 21 or 22)
    if len(upc) not in {21, 22}:
        return False

    # Convert UPC string to a list of integers
    upc_digits = [int(digit) for digit in upc]
    
    # Calculate OddSum and EvenSum
    odd_sum = sum(upc_digits[0::2]) * 3
    even_sum = sum(upc_digits[1::2][:-1] if len(upc) == 22 else upc_digits[1::2])
    
    # Calculate the checksum digit
    total_sum = odd_sum + even_sum
    checksum = (10 - (total_sum % 10)) % 10
    
    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # If UPC is 22 digits, validate checksum
    return checksum == upc_digits[-1] if len(upc) == 22 else str(checksum)


def get_usps_checksum(upc):
    upc = str(upc)
    return validate_usps_checksum(upc, True)


def fix_usps_checksum(upc):
    upc = str(upc)
    if(len(upc) > 21):
        fix_matches = re.findall("^(\\d{21})", upc)
        upc = fix_matches[0]
    return upc+str(get_usps_checksum(upc))


'''
// Get UPS Checkdigit and Info by stebo0728 and HolidayBows
// Source: http://www.codeproject.com/Articles/21224/Calculating-the-UPS-Tracking-Number-Check-Digit
// Source: http://www.codeproject.com/Articles/21224/Calculating-the-UPS-Tracking-Number-Check-Digit?msg=2961884#xx2961884xx
'''


def validate_ups_checksum(upc, return_check=False):
    upc = str(upc).upper()
    
    # Check for "1Z" prefix and extract remaining 16 characters if present
    if not upc.startswith("1Z"):
        return False
    upc = upc[2:]
    
    # Trim UPC to 16 characters if too long
    if len(upc) > 16:
        upc = re.findall("^\\w{16}", upc)[0]
    
    # Check valid length (should be 15 or 16 after prefix removal)
    if len(upc) not in {15, 16}:
        return False

    # Calculate OddSum and EvenSum based on character positions
    odd_sum = sum((int(ch) if ch.isdigit() else (ord(ch) - 63) % 10) for ch in upc[::2])
    even_sum = sum((int(ch) * 2 if ch.isdigit() else ((ord(ch) - 63) % 10) * 2) for ch in upc[1::2][:7])

    # Calculate CheckSum
    total_sum = odd_sum + even_sum
    checksum = (10 - (total_sum % 10)) % 10
    
    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # Validate checksum if UPC length is 16
    return checksum == int(upc[-1]) if len(upc) == 16 else str(checksum)


def get_ups_checksum(upc):
    upc = str(upc)
    return validate_ups_checksum(upc, True)


def fix_ups_checksum(upc):
    upc = str(upc)
    if(len(upc) > 17):
        fix_matches = re.findall("^(\\w{17})", upc)
        upc = fix_matches[0]
    return upc+str(get_ups_checksum(upc))


'''
// Get FEDEX Checkdigit by jbf777-ga
// Source: http://answers.google.com/answers/threadview/id/207899.html
'''


def validate_fedex_checksum(upc, return_check=False):
    upc = str(upc)
    
    # Trim UPC to 12 digits if it's too long
    if len(upc) > 12:
        upc = re.findall("^\\d{12}", upc)[0]
    
    # Check valid length (should be 11 or 12)
    if len(upc) not in {11, 12}:
        return False

    # Convert UPC string to a list of integers
    upc_digits = [int(digit) for digit in upc]
    
    # Calculate OddSum and EvenSum using specific weight pattern
    odd_sum = sum(upc_digits[i] * weight for i, weight in zip(range(0, 11, 2), [3, 7, 1, 3, 7, 1]))
    even_sum = sum(upc_digits[i] * weight for i, weight in zip(range(1, 10, 2), [1, 3, 7, 1, 3]))
    
    # Calculate the checksum digit using modulo 11
    total_sum = odd_sum + even_sum
    checksum = total_sum % 11
    
    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # Validate checksum if UPC length is 12
    return checksum == upc_digits[-1] if len(upc) == 12 else str(checksum)


def get_fedex_checksum(upc):
    upc = str(upc)
    return validate_fedex_checksum(upc, True)


def fix_fedex_checksum(upc):
    upc = str(upc)
    if(len(upc) > 11):
        fix_matches = re.findall("^(\\d{11})", upc)
        upc = fix_matches[0]
    return upc+str(get_fedex_checksum(upc))


'''
// IMEI (International Mobile Station Equipment Identity)
// Source: http://en.wikipedia.org/wiki/IMEI#Check_digit_computation
'''


def validate_imei_checksum(upc, return_check=False):
    upc = str(upc)
    
    # Trim UPC to 15 digits if it's too long
    if len(upc) > 15:
        upc = re.findall("^\\d{15}", upc)[0]
    
    # Check valid length (should be 14 or 15)
    if len(upc) not in {14, 15}:
        return False

    # Convert UPC string to a list of integers
    upc_digits = [int(digit) for digit in upc]
    
    # Calculate UPC sum for Luhn algorithm with alternating digit root calculation
    total_sum = sum(upc_digits[i] if i % 2 == 0 else get_digital_root(upc_digits[i] * 2) for i in range(14))
    
    # Calculate the checksum digit to make the total sum a multiple of 10
    checksum = (10 - (total_sum % 10)) % 10
    
    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # Validate checksum if UPC length is 15
    return checksum == upc_digits[-1] if len(upc) == 15 else str(checksum)


def get_imei_checksum(upc):
    upc = str(upc)
    return validate_imei_checksum(upc, True)


def fix_imei_checksum(upc):
    upc = str(upc)
    if(len(upc) > 14):
        fix_matches = re.findall("^(\\d{14})", upc)
        upc = fix_matches[0]
    return upc+str(get_imei_checksum(upc))


'''
// Bank Card Numbers
// Source: http://tywkiwdbi.blogspot.com/2012/06/checksum-number-on-credit-card.html
// Source: http://en.wikipedia.org/wiki/Luhn_algorithm#Implementation_of_standard_Mod_10
'''



def validate_bcn_checksum(upc, return_check=False):
    # Remove hyphens and spaces
    upc = re.sub("[-\\s]", "", str(upc))
    
    # Trim UPC to 16 digits if it's too long
    if len(upc) > 16:
        upc = re.findall(r"^\\d{16}", upc)[0]
    
    # Check valid length (should be 15 or 16)
    if len(upc) not in {15, 16}:
        return False

    # Convert UPC string to a list of integers
    upc_digits = [int(digit) for digit in upc]
    
    # Calculate UPC sum with alternating digital root for Luhn algorithm
    total_sum = sum(get_digital_root(upc_digits[i] * 2) if i % 2 == 0 else upc_digits[i] for i in range(15))
    
    # Calculate the checksum digit to make the total sum a multiple of 10
    checksum = (10 - (total_sum % 10)) % 10
    
    # Return the checksum digit if requested
    if return_check:
        return str(checksum)
    
    # Validate checksum if UPC length is 16
    return checksum == upc_digits[-1] if len(upc) == 16 else str(checksum)


def get_bcn_checksum(upc):
    upc = str(upc)
    upc = upc.replace("-", "")
    upc = upc.replace(" ", "")
    return validate_bcn_checksum(upc, True)


def fix_bcn_checksum(upc):
    upc = str(upc)
    upc = upc.replace("-", "")
    upc = upc.replace(" ", "")
    if(len(upc) > 15):
        fix_matches = re.findall("^(\\d{15})", upc)
        upc = fix_matches[0]
    return upc+str(get_bcn_checksum(upc))


'''
// Code 11
// Source: http://www.barcodeisland.com/code11.phtml
// Source: http://en.wikipedia.org/wiki/Code_11
'''


def get_code11_alt_checksum(upc):
    # Validate input
    if len(upc) < 1 or not re.match("^[0-9\\-]+$", upc):
        return False

    # Code11 mapping for values and characters
    code11_array = {str(i): i for i in range(10)}
    code11_array["-"] = 10

    def calculate_weighted_sum(upc_reversed, max_weight):
        """Calculate the weighted sum with a max weight limit."""
        return sum(((i % max_weight) + 1) * code11_array[char] for i, char in enumerate(upc_reversed))

    # First checksum calculation with max weight of 10
    upc_reversed = upc[::-1]
    checksum1 = list(code11_array.keys())[calculate_weighted_sum(upc_reversed, 10) % 11]

    # Second checksum calculation with max weight of 9
    checksum2 = list(code11_array.keys())[calculate_weighted_sum(upc_reversed, 9) % 11]

    # Return concatenated checksums
    return checksum1 + checksum2


'''
// Code 11
// Source: http://www.barcodeisland.com/code11.phtml
// Source: http://en.wikipedia.org/wiki/Code_11
'''


def get_code11_checksum(upc):
    # Validate input
    if len(upc) < 1 or not re.match("^[0-9\\-]+$", upc):
        return False

    # Code11 mapping for values and characters
    code11_array = {str(i): i for i in range(10)}
    code11_array["-"] = 10

    # Reverse the UPC and calculate the weighted sum
    upc_reversed = upc[::-1]
    upc_sum = sum((i % 10 + 1) * code11_array[char] for i, char in enumerate(upc_reversed))

    # Calculate and return the checksum
    checksum = list(code11_array.keys())[upc_sum % 11]
    return str(checksum)


'''
// Code 39
// Source: http://www.barcodeisland.com/code39.phtml
// Source: http://en.wikipedia.org/wiki/Code_39
'''


# Mapping for Code39 characters
CODE39_VALUES = {str(i): i for i in range(10)}
CODE39_VALUES.update({chr(65 + i): 10 + i for i in range(26)})  # A-Z
CODE39_VALUES.update({"-": 36, ".": 37, " ": 38, "$": 39, "/": 40, "+": 41, "%": 42})
CODE39_ARRAY = {v: k for k, v in CODE39_VALUES.items()}

def calculate_code39_checksum(upc, mod_value):
    # Validate input
    if len(upc) < 1 or not re.match("^[0-9A-Z\\-.\\$\\/\\+% ]+$", upc, re.IGNORECASE):
        return False

    # Calculate checksum based on mod_value
    upc_sum = sum(CODE39_VALUES[char] for char in upc.upper())
    checksum = CODE39_ARRAY[upc_sum % mod_value]
    return checksum

def get_code39_checksum_mod10(upc):
    return calculate_code39_checksum(upc, 10)

def get_code39_checksum_mod43(upc):
    return calculate_code39_checksum(upc, 43)

def get_code39_checksum(upc, getmod="43"):
    # Determine modulus based on getmod argument
    mod_value = 10 if getmod == "10" else 43
    return calculate_code39_checksum(upc, mod_value)


'''
// Code 93
// Source: http://www.barcodeisland.com/code93.phtml
// Source: http://en.wikipedia.org/wiki/Code_93
'''

# Mapping for Code93 characters and values
CODE93_VALUES = {str(i): i for i in range(10)}
CODE93_VALUES.update({chr(65 + i): 10 + i for i in range(26)})  # A-Z
CODE93_VALUES.update({"-": 36, ".": 37, " ": 38, "$": 39, "/": 40, "+": 41, "%": 42, "($)": 43, "(%)": 44, "(/)": 45, "(+)": 46})
CODE93_ARRAY = {v: k for k, v in CODE93_VALUES.items()}

def get_code93_alt_checksum(upc):
    # Validate input
    if len(upc) < 1 or not re.match("^[0-9A-Z\\-\\.\\$\\/\\+% ]+$", upc, re.IGNORECASE):
        return False

    # Helper function to calculate weighted sum with specified max weight
    def calculate_weighted_sum(upc_reversed, max_weight):
        return sum(((i % max_weight) + 1) * CODE93_VALUES[char] for i, char in enumerate(upc_reversed))

    # Calculate first checksum with max weight of 20
    upc_reversed = upc[::-1].upper()
    checksum1 = CODE93_ARRAY[calculate_weighted_sum(upc_reversed, 20) % 47]

    # Calculate second checksum with max weight of 15
    checksum2 = CODE93_ARRAY[calculate_weighted_sum(upc_reversed, 15) % 47]

    # Return concatenated checksums
    return checksum1 + checksum2


'''
// Code 93
// Source: http://www.barcodeisland.com/code93.phtml
// Source: http://en.wikipedia.org/wiki/Code_93
'''


def get_code93_checksum(upc):
    # Validate input
    if len(upc) < 1 or not re.match(r"^[0-9A-Z\\-\\.\\$\\/\\+% ]+$", upc, re.IGNORECASE):
        return False

    # Calculate the weighted sum with a maximum weight of 20
    upc_reversed = upc[::-1].upper()
    total_sum = sum(((i % 20) + 1) * CODE93_VALUES[char] for i, char in enumerate(upc_reversed))
    
    # Calculate the checksum character based on modulus 47
    checksum = CODE93_ARRAY[total_sum % 47]
    return checksum


'''
// Code 128
// Source: http://www.barcodeisland.com/code128.phtml
// Source: http://en.wikipedia.org/wiki/Code_128
'''


def get_code128_checksum(upc):
    # Validate input: hexadecimal string with even length and at least 6 characters
    if len(upc) < 6 or len(upc) % 2 != 0 or not re.match("^[0-9a-f]+$", upc, re.IGNORECASE):
        return False
    
    # Convert to lowercase and split into two-character hexadecimal parts
    upc_matches = re.findall("[0-9a-f]{2}", upc.lower())
    upc_to_dec = [int(x, 16) for x in upc_matches]
    
    # Calculate checksum with weighted sum
    checksum = upc_to_dec[0] if 102 < upc_to_dec[0] < 106 else 0
    for i, val in enumerate(upc_to_dec[1:], start=1):
        if not (i == len(upc_to_dec) - 1 and 105 < val < 108):  # Ignore last character if special code
            checksum += val * i

    # Return checksum as 2-digit hexadecimal string
    return format(checksum % 103, '02x')

def get_code128alt_checksum(upc):
    # Convert ASCII to hexadecimal and calculate checksum
    hex_upc = convert_ascii_code128_to_hex_code128(str(upc))
    return get_code128_checksum(hex_upc)

def get_code128dec_checksum(upc):
    # Validate input for decimal Code128 representation
    if len(upc) < 12 or not re.match("^\\d{3}+$", upc):
        return False
    
    # Convert each 3-digit decimal segment to 2-digit hexadecimal
    upc_hex = ''.join(format(int(upc[i:i+3]), '02x') for i in range(0, len(upc), 3))
    return get_code128_checksum(upc_hex)


'''
// MSI (Modified Plessey)
// Source: http://www.barcodeisland.com/msi.phtml
// Source: http://en.wikipedia.org/wiki/MSI_Barcode
'''


def get_msi_checksum_mod10(upc):
    upc_digits = [int(digit) for digit in str(upc)]
    
    # Split digits into odd and even indexed lists based on length
    odd_index_digits = upc_digits[1::2] if len(upc_digits) % 2 == 0 else upc_digits[::2]
    even_index_digits = upc_digits[::2] if len(upc_digits) % 2 == 0 else upc_digits[1::2]
    
    # Double odd indexed digits, split and sum
    odd_sum = sum(int(digit) for digit in ''.join(str(int(num) * 2) for num in odd_index_digits))
    even_sum = sum(even_index_digits)
    
    # Calculate and return the checksum
    total_sum = odd_sum + even_sum
    checksum = (10 - (total_sum % 10)) % 10
    return str(checksum)

def get_msi_checksum_mod11(upc, modtype="ibm"):
    # Set weight sequence based on modtype
    weights = (2, 3, 4, 5, 6, 7, 8, 9) if modtype.lower() == "ncr" else (2, 3, 4, 5, 6, 7)
    
    # Reverse UPC and calculate weighted sum
    upc_reversed = list(map(int, str(upc)[::-1]))
    weighted_sum = sum(num * weights[i % len(weights)] for i, num in enumerate(upc_reversed))
    
    # Calculate and return the checksum
    checksum = (11 - (weighted_sum % 11)) % 11
    return str(checksum)

def get_msi_checksum_mod1010(upc):
    # Apply mod 10 twice
    return get_msi_checksum_mod10(get_msi_checksum_mod10(upc))

def get_msi_checksum_mod1110(upc, modtype="ibm"):
    # Apply mod 11 then mod 10
    return get_msi_checksum_mod10(get_msi_checksum_mod11(upc, modtype))

def get_msi_checksum(upc, getmod="10", modtype="ibm"):
    # Select appropriate checksum calculation based on getmod value
    getmod = str(getmod)
    if getmod == "10":
        return get_msi_checksum_mod10(upc)
    elif getmod == "11":
        return get_msi_checksum_mod11(upc, modtype)
    elif getmod == "1010":
        return get_msi_checksum_mod1010(upc)
    elif getmod == "1110":
        return get_msi_checksum_mod1110(upc, modtype)
    return False


'''
// ISSN (International Standard Serial Number)
// Source: http://en.wikipedia.org/wiki/International_Standard_Serial_Number
'''


def validate_issn8_checksum(upc, return_check=False):
    # Remove hyphens and spaces
    upc = re.sub("[-\\s]", "", str(upc))
    
    # Validate length
    if len(upc) not in {7, 8}:
        return False

    # Convert ISSN to a list of integers
    upc_digits = [int(digit) for digit in upc[:7]]
    
    # Calculate weighted sum based on ISSN-8 rules
    total_sum = sum(upc_digits[i] * (8 - i) for i in range(7))
    
    # Calculate the checksum digit
    checksum = (11 - (total_sum % 11)) % 11
    
    # Return the checksum digit if requested
    if return_check:
        return str(checksum)

    # If ISSN is 8 digits, validate checksum
    return checksum == int(upc[7]) if len(upc) == 8 else str(checksum)


def get_issn8_checksum(upc):
    upc = str(upc)
    upc = upc.replace("-", "")
    upc = upc.replace(" ", "")
    return validate_issn8_checksum(upc, True)


def fix_issn8_checksum(upc):
    upc = str(upc)
    upc = upc.replace("-", "")
    upc = upc.replace(" ", "")
    if(len(upc) > 7):
        fix_matches = re.findall("^(\\d{7})", upc)
        upc = fix_matches[0]
    return upc+str(get_issn8_checksum(upc))


def validate_issn13_checksum(upc, return_check=False):
    upc = str(upc)
    if(not re.findall("^977(\\d{9})", upc)):
        return False
    if(re.findall("^977(\\d{9})", upc)):
        return validate_ean13_checksum(upc, return_check)


def get_issn13_checksum(upc):
    upc = str(upc)
    return validate_issn13_checksum(upc, True)


def fix_issn13_checksum(upc):
    upc = str(upc)
    if(not re.findall("^977(\\d{9})", upc)):
        return False
    if(re.findall("^977(\\d{9})", upc)):
        return fix_ean13_checksum(upc)


'''
// ISBN (International Standard Book Number)
// Source: http://en.wikipedia.org/wiki/ISBN
'''


def validate_isbn10_checksum(upc, return_check=False):
    # Remove hyphens and spaces
    upc = re.sub("[-\\s]", "", str(upc))
    
    # Validate length
    if len(upc) not in {9, 10}:
        return False

    # Calculate weighted sum for the first 9 digits
    upc_digits = [int(digit) for digit in upc[:9]]
    total_sum = sum(upc_digits[i] * (10 - i) for i in range(9))
    
    # Calculate checksum
    checksum = (11 - (total_sum % 11)) % 11
    checksum_char = "X" if checksum == 10 else str(checksum)
    
    # Return the checksum if requested
    if return_check:
        return checksum_char

    # If ISBN is 10 digits, validate checksum
    return checksum_char == upc[-1] if len(upc) == 10 else checksum_char


def get_isbn10_checksum(upc):
    upc = str(upc)
    upc = upc.replace("-", "")
    upc = upc.replace(" ", "")
    return validate_isbn10_checksum(upc, True)


def fix_isbn10_checksum(upc):
    upc = str(upc)
    upc = upc.replace("-", "")
    upc = upc.replace(" ", "")
    if(len(upc) > 9):
        fix_matches = re.findall("^(\\d{9})", upc)
        upc = fix_matches[1]
    return upc+str(get_isbn10_checksum(upc))


def validate_isbn13_checksum(upc, return_check=False):
    upc = str(upc)
    if(not re.findall("^(97[8-9])(\\d{9})", upc)):
        return False
    if(re.findall("^(97[8-9])(\\d{9})", upc)):
        return validate_ean13_checksum(upc, return_check)


def get_isbn13_checksum(upc):
    upc = str(upc)
    return validate_isbn13_checksum(upc, True)


def fix_isbn13_checksum(upc):
    upc = str(upc)
    if(not re.findall("^(97[8-9])(\\d{9})", upc)):
        return False
    if(re.findall("^(97[8-9])(\\d{9})", upc)):
        return fix_ean13_checksum(upc)


'''
// ISMN (International Standard Music Number)
// Source: http://en.wikipedia.org/wiki/International_Standard_Music_Number
// Source: http://www.ismn-international.org/whatis.html
// Source: http://www.ismn-international.org/manual_1998/chapter2.html
'''


def validate_ismn10_checksum(upc, return_check=False):
    # Remove "M" prefix, hyphens, and spaces
    upc = re.sub("[M-\\s]", "", str(upc))
    
    # Validate length (should be 8 or 9 digits after cleaning)
    if len(upc) not in {8, 9}:
        return False

    # Convert ISMN to a list of integers
    upc_digits = [int(digit) for digit in upc[:8]]
    
    # Calculate weighted sum with alternating weights 1 and 3, starting with 3 for 'M'
    total_sum = 3 + sum(upc_digits[i] * (3 if i % 2 == 1 else 1) for i in range(8))
    
    # Calculate checksum
    checksum = (10 - (total_sum % 10)) % 10
    
    # Return the checksum if requested
    if return_check:
        return str(checksum)

    # If ISMN is 9 digits, validate checksum
    return checksum == int(upc[8]) if len(upc) == 9 else str(checksum)


def get_ismn10_checksum(upc):
    upc = str(upc)
    upc = upc.replace("M", "")
    upc = upc.replace("-", "")
    upc = upc.replace(" ", "")
    return validate_ismn10_checksum(upc, True)


def fix_ismn10_checksum(upc):
    upc = str(upc)
    upc = upc.replace("M", "")
    upc = upc.replace("-", "")
    upc = upc.replace(" ", "")
    if(len(upc) > 9):
        fix_matches = re.findall("^(\\d{9})", upc)
        upc = fix_matches[1]
    return upc+str(get_ismn10_checksum(upc))


def validate_ismn13_checksum(upc, return_check=False):
    upc = str(upc)
    if(not re.findall("^9790(\\d{8})", upc)):
        return False
    if(re.findall("^9790(\\d{8})", upc)):
        return validate_ean13_checksum(upc, return_check)


def get_ismn13_checksum(upc):
    upc = str(upc)
    return validate_ismn13_checksum(upc, True)


def fix_ismn13_checksum(upc):
    upc = str(upc)
    if(not re.findall("^9790(\\d{8})", upc)):
        return False
    if(re.findall("^9790(\\d{8})", upc)):
        return fix_ean13_checksum(upc)


'''
// Get variable weight price checksum
// Source: http://wiki.answers.com/Q/How_does_a_price_embedded_bar_code_work
// Source: http://en.wikipedia.org/wiki/Universal_Product_Code#Prefixes
// Source: http://barcodes.gs1us.org/GS1%20US%20BarCodes%20and%20eCom%20-%20The%20Global%20Language%20of%20Business.htm
'''


# Predefined number replacement tables
NUM_REP1 = [0, 2, 4, 6, 8, 9, 1, 3, 5, 7]
NUM_REP2 = [0, 3, 6, 9, 2, 5, 8, 1, 4, 7]
NUM_REP3 = [0, 5, 9, 4, 8, 3, 7, 2, 6, 1]

def validate_vw_price_checksum(price, return_check=False):
    # Zero-pad the price to ensure it is at least 4 digits
    price = price.zfill(4)
    
    # Validate and extract 4 or 5 digits
    if len(price) > 5:
        price = re.match("^\\d{5}", price).group(0)
    elif len(price) not in {4, 5}:
        return False
    
    # Convert to list of integers
    price_digits = [int(digit) for digit in price]

    # Calculate the checksum using predefined tables
    if len(price) == 4:
        price_digits[0] = NUM_REP1[price_digits[0]]
        price_digits[1] = NUM_REP1[price_digits[1]]
        price_digits[2] = NUM_REP2[price_digits[2]]
        price_digits[3] = NUM_REP3[price_digits[3]]
        price_sum = sum(price_digits) * 3
    elif len(price) == 5:
        price_digits[1] = NUM_REP1[price_digits[1]]
        price_digits[2] = NUM_REP1[price_digits[2]]
        price_digits[3] = NUM_REP2[price_digits[3]]
        price_digits[4] = NUM_REP3[price_digits[4]]
        price_sum = sum(price_digits[1:]) * 3

    # Calculate the checksum digit
    checksum = price_sum % 10
    
    # Return the checksum if requested
    if return_check:
        return str(checksum)
    
    # Validate checksum for a 5-digit price
    return checksum == price_digits[0] if len(price) == 5 else str(checksum)

def get_vw_price_checksum(price):
    # Calculate the checksum for a given price
    return validate_vw_price_checksum(price.zfill(4), return_check=True)

def fix_vw_price_checksum(price):
    # Zero-pad and validate the price length
    price = price.zfill(4)
    if len(price) > 4:
        price = re.match("^\\d{4}", price).group(0)
    # Return the correct price with checksum prepended
    return str(get_vw_price_checksum(price)) + price

# Define a list of allowed prefixes for varying weight codes
VARYING_WEIGHT_PREFIXES = ["02"] + ["2{}".format(i) for i in range(1, 10)]

def validate_ean13_varying_weight_checksum(ean_code, return_check=False):
    # Remove any non-digit characters
    ean_code = re.sub("\\D", "", str(ean_code))
    
    # Check if the length is 13 and starts with a valid varying weight prefix
    if len(ean_code) != 13 or ean_code[:2] not in VARYING_WEIGHT_PREFIXES:
        return False

    # Convert code to a list of integers
    digits = [int(d) for d in ean_code[:12]]  # Exclude the checksum digit for calculation
    
    # Calculate the weighted sum for checksum (odd positions * 1, even positions * 3)
    total_sum = sum(d * (3 if i % 2 else 1) for i, d in enumerate(digits))
    checksum = (10 - (total_sum % 10)) % 10  # Calculate checksum digit
    
    # Return the checksum if requested
    if return_check:
        return str(checksum)
    
    # Validate the checksum digit
    return checksum == int(ean_code[-1])

def get_ean13_varying_weight_checksum(ean_code):
    # Returns the calculated checksum for an EAN-13 code with varying weight prefix
    return validate_ean13_varying_weight_checksum(ean_code, return_check=True)

def fix_ean13_varying_weight_checksum(ean_code):
    # Remove non-digit characters and ensure it is a 12-digit code
    ean_code = re.sub("\\D", "", str(ean_code)).zfill(12)
    
    # Return the complete code with the correct checksum appended
    return ean_code + get_ean13_varying_weight_checksum(ean_code)



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: prepygame2.py - Last Update: 11/15/2024 Ver. 2.12.0 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals

import pygame
import os
import re
import upcean.fonts

# Compatibility for Python 2 and 3
try:
    import pkg_resources
    pkgres = True
except ImportError:
    pkgres = False

try:
    basestring
except NameError:
    basestring = str

# Set up the display
screen_width, screen_height = 800, 600
screen = pygame.display.set_mode((screen_width, screen_height), pygame.RESIZABLE)  # Using RESIZABLE flag as a Pygame 2 feature
pygame.display.set_caption("Pygame 2 Drawing Example")

# Load font paths from upcean.fonts
fontpathocra = upcean.fonts.fontpathocra
fontpathocraalt = upcean.fonts.fontpathocraalt
fontpathocrb = upcean.fonts.fontpathocrb
fontpathocrbalt = upcean.fonts.fontpathocrbalt
fontpath = upcean.fonts.fontpath

def snapCoords(x, y):
    """
    Snaps the coordinates to the nearest integer plus 0.5.

    Parameters:
    - x, y: Original coordinates.

    Returns:
    - (snapped_x, snapped_y)
    """
    snapped_x = round(x) + 0.5
    snapped_y = round(y) + 0.5
    return (snapped_x, snapped_y)

def drawColorRectangle(surface, x1, y1, x2, y2, color, filled=True):
    """
    Draws a filled or outlined rectangle on the given surface.

    Parameters:
    - surface: Pygame Surface object.
    - x1, y1: Top-left corner coordinates.
    - x2, y2: Bottom-right corner coordinates.
    - color: Tuple representing (R, G, B).
    - filled: If True, draws a filled rectangle. If False, draws only the outline.
    """
    rect = pygame.Rect(x1, y1, x2 - x1, y2 - y1)
    if filled:
        pygame.draw.rect(surface, color, rect)
    else:
        pygame.draw.rect(surface, color, rect, 1)  # 1 pixel border
    return True  # Optional

def drawColorLine(surface, x1, y1, x2, y2, width, color):
    """
    Draws a line on the given surface.

    Parameters:
    - surface: Pygame Surface object.
    - x1, y1: Starting coordinates.
    - x2, y2: Ending coordinates.
    - width: Line width (integer >= 1).
    - color: Tuple representing (R, G, B).
    """
    width = max(1, int(width))
    pygame.draw.line(surface, color, (x1, y1), (x2, y2), width)
    return True  # Optional

def drawColorText(surface, size, x, y, text, color, ftype="ocrb"):
    """
    Draws text on the given surface with the specified font and color.

    Parameters:
    - surface: Pygame Surface object.
    - size: Font size.
    - x, y: Position to draw the text.
    - text: The text string to render.
    - color: Tuple representing (R, G, B).
    - ftype: Font type ("ocra" or "ocrb").
    """
    if ftype == "ocra":
        try:
            font = pygame.font.Font(fontpathocra, size)
        except IOError:
            font = pygame.font.Font(fontpathocraalt, size)
    elif ftype == "ocrb":
        try:
            font = pygame.font.Font(fontpathocrb, size)
        except IOError:
            font = pygame.font.Font(fontpathocrbalt, size)
    else:
        font = pygame.font.Font(fontpath, size)  # Default font

    # Ensure text is a Unicode string in Python 2
    if isinstance(text, bytes):
        text = text.decode('utf-8')

    text_surface = font.render(text, True, color)
    surface.blit(text_surface, (x, y))
    return True

def drawColorRectangleAlt(surface, x1, y1, x2, y2, color):
    """
    Draws an outlined rectangle on the given surface.

    Parameters:
    - surface: Pygame Surface object.
    - x1, y1: Top-left corner coordinates.
    - x2, y2: Bottom-right corner coordinates.
    - color: Tuple representing (R, G, B).
    """
    rect = pygame.Rect(x1, y1, x2 - x1, y2 - y1)
    pygame.draw.rect(surface, color, rect, 1)  # 1 pixel border
    return True



# -*- coding: utf-8 -*-
'''
    This program is free software; you can redistribute it and/or modify
    it under the terms of the Revised BSD License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    Revised BSD License for more details.

    Copyright 2011-2023 Game Maker 2k - https://github.com/GameMaker2k
    Copyright 2011-2023 Kazuki Przyborowski - https://github.com/KazukiPrzyborowski

    $FileInfo: stf.py - Last Update: 11/15/2024 Ver. 2.12.0 RC 1 - Author: cooldude2k $
'''

from __future__ import absolute_import, division, print_function, unicode_literals
from upcean.encode.predraw import *
import re
import upcean.encode.predraw.getsfname
import upcean.support
try:
    from PIL import Image, UnidentifiedImageError
    hasuie = True
except ImportError:
    from PIL import Image
    hasuie = False
try:
    from io import StringIO, BytesIO
except ImportError:
    try:
        from cStringIO import StringIO
        from cStringIO import StringIO as BytesIO
    except ImportError:
        from StringIO import StringIO
        from StringIO import StringIO as BytesIO
pilsupport = upcean.support.check_for_pil()
pillowsupport = upcean.support.check_for_pillow()
cairosupport = upcean.support.check_for_cairo()
if(cairosupport):
    import cairo


def decode_stf_barcode(infile="./stf.png", resize=1, barheight=(48, 54), barwidth=(1, 1), shiftcheck=False, shiftxy=(0, 0), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), locatebarcode=False, imageoutlib="pillow"):
    if(not re.findall(r"^([0-9]*[\.]?[0-9])", str(resize)) or int(resize) < 1):
        resize = 1
    if(isinstance(infile, Image.Image)):
        upc_img = infile.convert('RGB')
    elif(cairosupport and isinstance(infile, cairo.ImageSurface)):
        if(sys.version[0] == "2"):
            stdoutfile = StringIO()
        if(sys.version[0] >= "3"):
            stdoutfile = BytesIO()
        infile.write_to_png(stdoutfile)
        stdoutfile.seek(0)
        upc_img = Image.open(stdoutfile).convert('RGB')
    else:
        try:
            infile.seek(0)
            if(hasuie):
                try:
                    upc_img = Image.open(infile).convert('RGB')
                except UnidentifiedImageError:
                    return False
                    '''upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), infile.read());'''
            else:
                try:
                    upc_img = Image.open(infile).convert('RGB')
                except IOError:
                    return False
                    '''upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), infile.read());'''
        except AttributeError:
            if(hasuie):
                try:
                    upc_img = Image.open(infile).convert('RGB')
                except UnidentifiedImageError:
                    return False
                    '''prefile = open(infile, "rb");
     upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), prefile.read());
     prefile.close();'''
            else:
                try:
                    upc_img = Image.open(infile).convert('RGB')
                except IOError:
                    return False
                    '''prefile = open(infile, "rb");
     upc_img = Image.frombytes("RGB", (((115 * barwidth[0]) ) * int(resize), (barheight[1] + 9) * int(resize)), prefile.read());
     prefile.close();'''
    threewidebar = True
    barsize = barwidth[0] * int(resize)
    starty = (int(upc_img.size[1] / 2) - ((barwidth[1] - 1) * 9)) + shiftxy[1]
    if(threewidebar):
        barcode_dict = {'10101110111010': "0", '11101010101110': "1", '10111010101110': "2", '11101110101010': "3", '10101110101110': "4",
                        '11101011101010': "5", '10111011101010': "6", '10101011101110': "7", '11101010111010': "8", '10111010111010': "9"}
    else:
        barcode_dict = {'10101110111010': "0", '11101010101110': "1", '10111010101110': "2", '11101110101010': "3", '10101110101110': "4",
                        '11101011101010': "5", '10111011101010': "6", '10101011101110': "7", '11101010111010': "8", '10111010111010': "9"}
    startx = 17
    if(shiftcheck):
        prestartx = shiftxy[0]
        startx = shiftxy[0]
        gotvalue = False
        barcodesize = 0
        while(prestartx < upc_img.size[0]):
            inprestartx = prestartx
            substartx = prestartx + (8 * (barwidth[0] * int(resize)))
            curpixelist = []
            if(upc_img.getpixel((inprestartx, starty)) == barcolor[0]):
                if(inprestartx+(8 * (barwidth[0] * int(resize))) > upc_img.size[0]):
                    return False
                if(threewidebar):
                    icount = 0
                    imaxc = 8
                    while(icount < imaxc):
                        curpixelist.append(upc_img.getpixel(
                            (inprestartx+(icount * (barwidth[0] * int(resize))), starty)))
                        icount += 1
                    if((curpixelist[0] == barcolor[0] and curpixelist[1] == barcolor[0] and curpixelist[2] == barcolor[2] and curpixelist[3] == barcolor[0] and curpixelist[4] == barcolor[0] and curpixelist[5] == barcolor[2] and curpixelist[6] == barcolor[0] and curpixelist[7] == barcolor[2])):
                        preinprestartx = inprestartx + \
                            (8 * (barwidth[0] * int(resize)))
                        precurpixelist = []
                        while(preinprestartx < upc_img.size[0]):
                            precurpixelist = []
                            if((preinprestartx+(14 * (barwidth[0] * int(resize))) > upc_img.size[0])):
                                return False
                            icount = 0
                            imaxc = 14
                            while(icount < imaxc):
                                precurpixelist.append(upc_img.getpixel(
                                    (preinprestartx+(icount * (barwidth[0] * int(resize))), starty)))
                                icount += 1
                            preinprestartx += (14 *
                                               (barwidth[0] * int(resize)))
                            barcodesize += 1
                            if((precurpixelist[0] == barcolor[0] and precurpixelist[1] == barcolor[0] and precurpixelist[2] == barcolor[2] and precurpixelist[3] == barcolor[0] and precurpixelist[4] == barcolor[2] and precurpixelist[5] == barcolor[0] and precurpixelist[6] == barcolor[0] and precurpixelist[7] == barcolor[2] and precurpixelist[8] == barcolor[2])):
                                break
                        barcodesize = barcodesize - 1
                        inprestartx += (8 + (barcodesize * 14)) * \
                            (barwidth[0] * int(resize))
                        icount = 0
                        imaxc = 7
                        while(icount < imaxc):
                            curpixelist.append(upc_img.getpixel(
                                (inprestartx+(icount * (barwidth[0] * int(resize))), starty)))
                            icount += 1
                        if((curpixelist[0] == barcolor[0] and curpixelist[1] == barcolor[0] and curpixelist[2] == barcolor[2] and curpixelist[3] == barcolor[0] and curpixelist[4] == barcolor[0] and curpixelist[5] == barcolor[2] and curpixelist[6] == barcolor[0] and curpixelist[7] == barcolor[2]) and (curpixelist[8] == barcolor[0] and curpixelist[9] == barcolor[0] and curpixelist[10] == barcolor[2] and curpixelist[11] == barcolor[0] and curpixelist[12] == barcolor[2] and curpixelist[13] == barcolor[0] and curpixelist[14] == barcolor[0])):
                            startx = substartx
                            break
                else:
                    icount = 0
                    imaxc = 8
                    while(icount < imaxc):
                        curpixelist.append(upc_img.getpixel(
                            (inprestartx+(icount * (barwidth[0] * int(resize))), starty)))
                        icount += 1
                    if((curpixelist[0] == barcolor[0] and curpixelist[1] == barcolor[0] and curpixelist[2] == barcolor[2] and curpixelist[3] == barcolor[0] and curpixelist[4] == barcolor[0] and curpixelist[5] == barcolor[2] and curpixelist[6] == barcolor[0] and curpixelist[7] == barcolor[2])):
                        preinprestartx = inprestartx + \
                            (8 * (barwidth[0] * int(resize)))
                        precurpixelist = []
                        while(preinprestartx < upc_img.size[0]):
                            precurpixelist = []
                            if((preinprestartx+(12 * (barwidth[0] * int(resize))) > upc_img.size[0])):
                                return False
                            icount = 0
                            imaxc = 12
                            while(icount < imaxc):
                                precurpixelist.append(upc_img.getpixel(
                                    (preinprestartx+(icount * (barwidth[0] * int(resize))), starty)))
                                icount += 1
                            preinprestartx += (12 *
                                               (barwidth[0] * int(resize)))
                            barcodesize += 1
                            if((precurpixelist[0] == barcolor[0] and precurpixelist[1] == barcolor[0] and precurpixelist[2] == barcolor[2] and precurpixelist[3] == barcolor[0] and precurpixelist[4] == barcolor[2] and precurpixelist[5] == barcolor[0] and precurpixelist[6] == barcolor[0] and precurpixelist[7] == barcolor[2] and precurpixelist[8] == barcolor[2])):
                                break
                        barcodesize = barcodesize - 1
                        inprestartx += (8 + (barcodesize * 12)) * \
                            (barwidth[0] * int(resize))
                        icount = 0
                        imaxc = 7
                        while(icount < imaxc):
                            curpixelist.append(upc_img.getpixel(
                                (inprestartx+(icount * (barwidth[0] * int(resize))), starty)))
                            icount += 1
                        if((curpixelist[0] == barcolor[0] and curpixelist[1] == barcolor[0] and curpixelist[2] == barcolor[2] and curpixelist[3] == barcolor[0] and curpixelist[4] == barcolor[0] and curpixelist[5] == barcolor[2] and curpixelist[6] == barcolor[0] and curpixelist[7] == barcolor[2]) and (curpixelist[8] == barcolor[0] and curpixelist[9] == barcolor[0] and curpixelist[10] == barcolor[2] and curpixelist[11] == barcolor[0] and curpixelist[12] == barcolor[2] and curpixelist[13] == barcolor[0] and curpixelist[14] == barcolor[0])):
                            startx = substartx
                            break
            prestartx += 1
        shiftxy = (0, shiftxy[1])
    else:
        startx = ((21 * (barwidth[0] * int(resize))) + shiftxy[0])
        preinprestartx = startx
        precurpixelist = []
        barcodesize = 0
        while(preinprestartx < upc_img.size[0]):
            precurpixelist = []
            if(threewidebar):
                if((preinprestartx+(14 * (barwidth[0] * int(resize))) > upc_img.size[0])):
                    return False
                icount = 0
                imaxc = 14
                while(icount < imaxc):
                    precurpixelist.append(upc_img.getpixel(
                        (preinprestartx+(icount * (barwidth[0] * int(resize))), starty)))
                    icount += 1
                preinprestartx += (14 * (barwidth[0] * int(resize)))
                barcodesize += 1
                if((precurpixelist[0] == barcolor[0] and precurpixelist[1] == barcolor[0] and precurpixelist[2] == barcolor[2] and precurpixelist[3] == barcolor[0] and precurpixelist[4] == barcolor[2] and precurpixelist[5] == barcolor[0] and precurpixelist[6] == barcolor[0] and precurpixelist[7] == barcolor[2] and precurpixelist[8] == barcolor[2])):
                    break
            else:
                if((preinprestartx+(12 * (barwidth[0] * int(resize))) > upc_img.size[0])):
                    return False
                icount = 0
                imaxc = 12
                while(icount < imaxc):
                    precurpixelist.append(upc_img.getpixel(
                        (preinprestartx+(icount * (barwidth[0] * int(resize))), starty)))
                    icount += 1
                preinprestartx += (12 * (barwidth[0] * int(resize)))
                barcodesize += 1
                if((precurpixelist[0] == barcolor[0] and precurpixelist[1] == barcolor[0] and precurpixelist[2] == barcolor[2] and precurpixelist[3] == barcolor[0] and precurpixelist[4] == barcolor[2] and precurpixelist[5] == barcolor[0] and precurpixelist[6] == barcolor[0] and precurpixelist[7] == barcolor[2] and precurpixelist[8] == barcolor[2])):
                    break
        barcodesize = barcodesize - 1
    endx = int(startx + ((barcodesize * 14) * (barwidth[0] * int(resize))))
    if(threewidebar):
        if(locatebarcode):
            postendx = endx + (5 * (barwidth[0] * int(resize)))
        endx = int(startx + ((barcodesize * 14) * (barwidth[0] * int(resize))))
    else:
        if(locatebarcode):
            postendx = endx + (5 * (barwidth[0] * int(resize)))
        endx = int(startx + ((barcodesize * 12) * (barwidth[0] * int(resize))))
    if(locatebarcode):
        prestartx = startx - (8 * (barwidth[0] * int(resize)))
        endx -= (1 * (barwidth[0] * int(resize)))
        countyup = starty
        while(countyup >= 0):
            curonepixel = upc_img.getpixel((prestartx, countyup))
            curtwopixel = upc_img.getpixel(
                (prestartx + (2 * (barwidth[0] * int(resize))), countyup))
            if(curonepixel == barcolor[2] or curtwopixel == barcolor[0]):
                break
            countyup += 1
        countyup -= 1
        countydown = starty
        while(countydown <= upc_img.size[1]):
            curonepixel = upc_img.getpixel((prestartx, countydown))
            curtwopixel = upc_img.getpixel(
                (prestartx + (2 * (barwidth[0] * int(resize))), countydown))
            if(curonepixel == barcolor[2] or curtwopixel == barcolor[0]):
                break
            countydown -= 1
        countydown -= 1
        return ("stf", prestartx, startx, 0, 0, endx, countyup, round(countyup / 2), round(countydown * 2), countydown, countyup, barcodesize)
    startxalt = startx
    listcount = 0
    pre_upc_whole = []
    while(startxalt < endx):
        listcount = 0
        pre_upc_list = []
        while(listcount < 14):
            curpixel = upc_img.getpixel((startx, starty))
            if(curpixel == barcolor[0]):
                incount = 0
                inbarwidth = barwidth[0] - 1
                while(incount <= inbarwidth):
                    incurpixel = upc_img.getpixel((startx + incount, starty))
                    if(incurpixel != barcolor[0]):
                        return False
                    incount += 1
                pre_upc_list.append("1")
            if(curpixel == barcolor[2]):
                incount = 0
                inbarwidth = barwidth[0] - 1
                while(incount <= inbarwidth):
                    incurpixel = upc_img.getpixel((startx + incount, starty))
                    if(incurpixel != barcolor[2]):
                        return False
                    incount += 1
                pre_upc_list.append("0")
            startx += 1 * (barwidth[0] * int(resize))
            startxalt += 1 * (barwidth[0] * int(resize))
            listcount += 1
        pre_upc_whole.append("".join(pre_upc_list))
    upc_img.close()
    countlist = len(pre_upc_whole)
    listcount = 0
    barcode_list = []
    while(listcount < countlist):
        barcode_value = barcode_dict.get(pre_upc_whole[listcount], False)
        if(not barcode_value):
            return False
        barcode_list.append(barcode_value)
        listcount += 1
        upc = "".join(barcode_list)
    return upc


def get_stf_barcode_location(infile="./stf.png", resize=1, barheight=(48, 54), barwidth=(1, 1), shiftcheck=False, shiftxy=(0, 0), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
    return decode_stf_barcode(infile, resize, barheight, barwidth, shiftcheck, shiftxy, barcolor, True, imageoutlib)


def decode_code25_barcode(infile="./code25.png", resize=1, barheight=(48, 54), barwidth=(1, 1), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), locatebarcode=False, imageoutlib="pillow"):
    return decode_stf_barcode(infile, resize, barheight, barwidth, barcolor, locatebarcode, imageoutlib)


def get_code25_barcode_location(infile="./code25.png", resize=1, barheight=(48, 54), barwidth=(1, 1), shiftcheck=False, shiftxy=(0, 0), barcolor=((0, 0, 0), (0, 0, 0), (255, 255, 255)), imageoutlib="pillow"):
    return decode_code25_barcode(infile, resize, barheight, barwidth, shiftcheck, shiftxy, barcolor, True, imageoutlib)
